'
' USB Generic HID descriptor module
'
' Do not change the order of the commands and directives
'
' Place any $defines here. i.e. before the loading of USB_MemAlloc.inc
' See USB_Defs.inc for the defines used
'
$define USB_VID                 $04D8       ' User VID value
$define USB_PID                 $0001       ' User PID value
$define HID_BUFFER_SIZE         64          ' USB Buffer Size. (HID EP1 can be 8, 16, 32 or 64)
$define USB_BUS_POWER           250         ' Bus power (50 to 250 milliAmps) x2

$define USB_SERVICE                         ' Use the Timer1 low priority interrupt for enumeration and bus servicing
$define USB_USE_HID                         ' Indicate we require to use a HID interface
$define USB_CONNECT_LED PORTC.3             ' This pin goes high when the USB is in a connected state
$define USB_SELF_POWER                      ' Indicate that the PICmicro is self powered. i.e. Not powered by the bus
$define HID_EP_OUT_POLLING_MS   1           ' HID - EP01 - 1ms (EP01Out)
$define HID_EP_IN_POLLING_MS    1           ' HID - EP01 - 1ms (EP01In)
'
'---------------------------------------------------------------------------------------------------------
'
    Include "USB_MemAlloc.inc"              ' ** Load the USB Stack and Dual Port RAM variable definitions **
'
'---------------------------------------------------------------------------------------------------------
'
    GoTo OverDescriptorTables               ' Jump over the descriptor tables
'
'---------------------------------------------------------------------------------------------------------
'
    $define HID_RPT01_SIZE 47               ' Size of "hid_rpt01" table
'
' This section may be modified to suit your requirements
' Do not change any of the label names, or the format of the tables
'
device_dsc:
    CData   SizeOf_Table,_                   ' Size of this descriptor (in bytes)
            USB_DESCRIPTOR_DEVICE,_          ' device descriptor type
            $02, $00,_                       ' USB Spec Release Number in BCD format (low byte, high byte)
            $00,_                            ' Class Code
            $00,_                            ' Subclass code
            $00,_                            ' Protocol code
            USB_EP0_BUFF_SIZE,_              ' Max packet size For EP0
            USB_VID & $FF, USB_VID >> 8,_    ' Vendor ID (low byte, byte byte)
            USB_PID & $FF, USB_PID >> 8,_    ' Product ID (low byte, high byte)
            USB_DEVICE_VERSION & $FF,_       ' Device release number in BCD format (low byte)
            USB_DEVICE_VERSION >> 8,_        ' Device release number in BCD format (high byte)
            USB_MANUFACTURER_INDEX,_         ' Manufacturer String index
            USB_PRODUCT_INDEX,_              ' Product String index
            USB_SERIAL_INDEX,_               ' Device serial number String index
            $01                              ' Number of possible configurations
'
' Configuration Descriptor
'
configDescriptor1:
    CData   $09,_                            ' Size of this section of the descriptor (in bytes)
            USB_DESCRIPTOR_CONFIGURATION,_   ' Configuration descriptor type
            SizeOf_Table, $00,_              ' Size Of "configDescriptor1" table - Total length of data (low byte, high byte)
            $01,_                            ' Number of interfaces in this cfg
            $01,_                            ' Index value of this configuration
            $00,_                            ' Configuration string index
            _DEFAULT | _RWU,_                ' Attributes, see USB_Defs.inc
            USB_BUS_POWER,_                  ' Max power consumption (2X mA)
            '
            ' Interface descriptor
            '
            $09,_                            ' Size of this section of the descriptor (in bytes)
            USB_DESCRIPTOR_INTERFACE,_       ' Interface descriptor type
            $00,_                            ' Interface Number
            $00,_                            ' Alternate Setting Number
            $02,_                            ' Number of endpoints in this intf
            HID_INTF,_                       ' Class code
            $00,_                            ' Subclass code
            $00,_                            ' Protocol code
            $00,_                            ' Interface string index
            '
            ' HID class-specific descriptor
            '
            $09,_                            ' Size of this section of the descriptor (in bytes)
            DSC_HID,_                        ' HID descriptor type
            $10, $01,_                       ' HID Spec Release Number in BCD format (low byte, high byte)
            $00,_                            ' Country Code (0x00 for Not supported)
            HID_NUM_OF_DSC,_                 ' Number of class descriptors
            DSC_RPT,_                        ' Report descriptor type
            HID_RPT01_SIZE, $00,_            ' Size of "hid_rpt01" table (low byte, high byte)
            '
            ' Endpoint descriptor (IN)
            '
            $07,_                            ' Size of this section of the descriptor (in bytes)
            USB_DESCRIPTOR_ENDPOINT,_        ' Endpoint descriptor type
            _EP01_IN,_                       ' Endpoint 1 In
            _INTERRUPT,_                     ' Interrupt transfers
            HID_INT_IN_EP_SIZE, $00,_        ' Maximum packet size (low byte, high byte)
            HID_EP_IN_POLLING_MS,_           ' Polling interval (milliseconds)
            '
            ' Enpoint descriptor (OUT)
            '
            $07,_                            ' Size of This section of the descriptor (in bytes)
            USB_DESCRIPTOR_ENDPOINT,_        ' Endpoint descriptor type
            _EP01_OUT,_                      ' Endpoint 1 Out
            _INTERRUPT,_                     ' Interrupt transfers
            HID_INT_OUT_EP_SIZE, $00,_       ' Maximum packet size (low byte, high byte)
            HID_EP_OUT_POLLING_MS            ' Polling interval (milliseconds)
'
' Language string - unicode format
'
sd000:
    CData   Byte SizeOf_Table,_             ' Size of this descriptor (in bytes)
            Byte USB_DESCRIPTOR_STRING,_    ' Descriptor Type
            Word $0904                      ' language ID - (high byte, low byte)
'
' Manufacturer string - unicode format
'
sd001:
    CData   Byte SizeOf_Table,_             ' Size of this descriptor (in bytes)
            Byte USB_DESCRIPTOR_STRING,_    ' Descriptor Type
            Word "Crownhill"
'
' Product string - unicode format
'
sd002:
    CData   Byte SizeOf_Table,_             ' Size of this descriptor (in bytes)
            Byte USB_DESCRIPTOR_STRING,_    ' Descriptor Type
            Word "Proton - USB HID"

$ifdef USB_HAVE_SERIAL_STRING
sd003:
    CData   Byte SizeOf_Table,_             ' Size of this descriptor (in bytes)
            Byte USB_DESCRIPTOR_STRING,_    ' Descriptor Type
            Word "1234567"                  ' Serial string - unicode format
$endif
'
' This example report descriptor for a generic HID defines one report of each type.
' Each report contains two bytes of data with a vendor-defined Usage.
'
hid_rpt01:
    CData   $06, $A0, $FF,_                 ' Usage page (vendor defined)
            $09, $01,_                      ' Usage ID (vendor defined)
            $A1, $01,_                      ' Collection (application)
            '
            ' The Input report:
            '
            $09, $03,_                      ' Usage ID - vendor defined
            $15, $00,_                      ' Logical Minimum (0)
            $26, $FF, $00,_                 ' Logical Maximum (255)
            $75, $08,_                      ' Report Size (8 bits)
            $95, HID_REPORT_BYTES_IN,_      ' Report Count (2 fields)
            $81, $02,_                      ' Input (Data, Variable, Absolute)
            '
            ' The Output report:
            '
            $09, $04,_                      ' Usage ID - vendor defined
            $15, $00,_                      ' Logical Minimum (0)
            $26, $FF, $00,_                 ' Logical Maximum (255)
            $75, $08,_                      ' Report Size (8 bits)
            $95, HID_REPORT_BYTES_OUT,_     ' Report Count (2 fields)
            $91, $02,_                      ' Output (Data, Variable, Absolute)
            '
            ' The Feature report:
            '
            $09, $05,_                      ' Usage ID - vendor defined
            $15, $00,_                      ' Logical Minimum (0)
            $26, $FF, $00,_                 ' Logical Maximum (255)
            $75, $08,_                      ' Report Size (8 bits)
            $95, $02,_                      ' Report Count (2 fields)
            $B1, $02,_                      ' Feature (Data, Variable, Absolute)
            $C0                             ' end collection
'
'---------------------------------------------------------------------------------------------------------
'
USB_CD_Pointer:
    CData configDescriptor1, configDescriptor1 ' Create a pointer to the descriptor's cfg tables

USB_SD_Pointer:
$ifdef USB_HAVE_SERIAL_STRING
    CData sd000, sd001, sd002, sd003        ' Create a pointer to the descriptor's sd tables(with serial)
$else
    CData sd000, sd001, sd002               ' Create a pointer to the descriptor's sd tables (without serial)
$endif

OverDescriptorTables:
    Include "USB_System.inc"                ' ** Load the USB routines into the program **

$ifdef USB_USE_HID
    Include "USB_UserHID.inc"               ' ** Load the User HID routines into the program (if a HID interface is being used)
$endif

