'
' Constants and defines used by the USB routines
'
$ifndef True
    $define True 1
$endif
$ifndef False
    $define False 0
$endif

'----------------------------------------------------------------------------
' User VID value
$ifndef USB_VID
    $define USB_VID                 $04D8
$endif
'----------------------------------------------------------------------------
' User PID value
$ifndef USB_PID
    $define USB_PID                 $000A
$endif
'----------------------------------------------------------------------------
' HID buffer size (in bytes) (8, 16, 32, or 64)
' If using a smaller PICmicro device, using a buffer size of 32 or 64 will allow
' the use of dual port RAM as the user buffer, thus saving normal RAM
'
$ifndef HID_BUFFER_SIZE
    $define HID_BUFFER_SIZE         64               ' Default HID EP1 can be 8, 16, 32 or 64
$endif
'----------------------------------------------------------------------------
' HID EP Polling (in milliseconds)
$ifndef HID_EP_OUT_POLLING_MS
    $define HID_EP_OUT_POLLING_MS   1               ' Default HID - EP01 - 1ms (EP01Out)
$endif

$ifndef HID_EP_IN_POLLING_MS
    $define HID_EP_IN_POLLING_MS    1               ' Default HID - EP01 - 1ms (EP01In)
$endif
'----------------------------------------------------------------------------
' Bus power (50 to 250 milliAmps) x2
$ifndef USB_BUS_POWER
    $define USB_BUS_POWER       50                  ' Default Bus power
$endif
'----------------------------------------------------------------------------
$ifndef MAX_EP_NUMBER
    $define MAX_EP_NUMBER       1                   ' Default UEP1 minimum, up to 16
$endif
'----------------------------------------------------------------------------
' Device version (device_dsc) in BCD format
$ifndef USB_DEVICE_VERSION
    $define USB_DEVICE_VERSION  $0110               ' Default Device version
$endif
'----------------------------------------------------------------------------
$ifndef USB_SERVICE_US
    $define USB_SERVICE_US          1000            ' Default USB service interval time (in uS)
$endif

$ifndef USB_SERVICE_ENUM_US
    $define USB_SERVICE_ENUM_US     60              ' Default USB enumeration interval servicing (in uS)
$endif
'----------------------------------------------------------------------------
' descriptor string index (device_desc)
$ifndef USB_MANUFACTURER_INDEX
    $define USB_MANUFACTURER_INDEX  1
$endif

$ifndef USB_PRODUCT_INDEX
    $define USB_PRODUCT_INDEX       2
$endif

$ifndef USB_SERIAL_INDEX
    $define USB_SERIAL_INDEX        0
$endif
'----------------------------------------------------------------------------
' Timeout between CONFIGURED_STATE and setting 'tUSB_Attached'
$ifndef USB_CONNECT_MS
    $define USB_CONNECT_MS          1200
$endif
'----------------------------------------------------------------------------
' The size of EndPoint0
$ifndef USB_EP0_BUFF_SIZE
    $define USB_EP0_BUFF_SIZE       8    ' Used to calculate RAM offsets
$endif
'----------------------------------------------------------------------------
$ifndef HID_REPORT_BYTES_IN
    $define HID_REPORT_BYTES_IN     HID_BUFFER_SIZE    ' HID Report bytes in
$endif

$ifndef HID_REPORT_BYTES_OUT
    $define HID_REPORT_BYTES_OUT    HID_BUFFER_SIZE    ' HID_Report bytes out
$endif

' endpoints allocation - see USBmmap.h for an explanation of how the endpoint allocation works
'USBcfg.h
$ifndef HID_INTF_ID
   $define HID_INTF_ID           $00
$endif

$ifndef HID_INT_OUT_EP_SIZE
   $define HID_INT_OUT_EP_SIZE   HID_REPORT_BYTES_IN
$endif

$ifndef HID_INT_IN_EP_SIZE
   $define HID_INT_IN_EP_SIZE    HID_REPORT_BYTES_OUT
$endif
'
' the number of bytes in each report,
' calculated from reportSize and reportCount in the report descriptor
'
$ifndef HID_NUM_OF_DSC
   $define HID_NUM_OF_DSC        $01
$endif

$ifndef HID_INPUT_REPORT_BYTES
   $define HID_INPUT_REPORT_BYTES    HID_REPORT_BYTES_IN
$endif

$ifndef HID_OUTPUT_REPORT_BYTES
   $define HID_OUTPUT_REPORT_BYTES   HID_REPORT_BYTES_OUT
$endif

$ifndef HID_FEATURE_REPORT_BYTES
   $define HID_FEATURE_REPORT_BYTES  2
$endif

' for tracking alternate setting
'USBcfg.h
$ifndef MAX_NUM_INT
   $define MAX_NUM_INT 1
$endif

'----------------------------------------------------------------------------
$if _USB_ram = 1024               ' Can we use the extra RAM in the USB portion as a buffer?
    $define USB_EXTENDED_RAM
$endif

   Symbol ReloadTimerValue      = 65536 - _xtal * (USB_SERVICE_US / 4)
   Symbol ReloadTimerValueEnum  = 65536 - _xtal * (USB_SERVICE_ENUM_US / 4)

' UCFG Initialisation Parameters
   Symbol _PPBM0 =     $00           ' Pingpong Buffer Mode 0
   Symbol _PPBM1 =     $01           ' Pingpong Buffer Mode 1
   Symbol _PPBM2 =     $02           ' Pingpong Buffer Mode 2
   Symbol _LS =        $00           ' Use Low-Speed USB Mode
   Symbol _FS =        $04           ' Use Full-Speed USB Mode
   Symbol _TRINT =     $00           ' Use internal transceiver
   Symbol _TREXT =     $08           ' Use external transceiver
   Symbol _PUEN =      $10           ' Use internal pull-up resistor
   Symbol _OEMON =     $40           ' Use SIE output indicator
   Symbol _UTEYE =     $80           ' Use Eye-Pattern test

' UEPn Initialisation Parameters
   Symbol EP_CTRL =    $06           ' Cfg Control pipe for this ep
   Symbol EP_OUT =     $0C           ' Cfg OUT only pipe for this ep
   Symbol EP_IN =      $0A           ' Cfg IN only pipe for this ep
   Symbol EP_OUT_IN =  $0E           ' Cfg both OUT & IN pipes for this ep
   Symbol HSHK_EN =    $10           ' Enable handshake packet
                                     ' Handshake should be disable for isoch
'
'   USB - PICmicro Endpoint Definitions
'   This is used when checking the value read from USTAT
'   Note: These definitions are not used in the descriptors.
'   EP addresses used in the descriptor s have different format and
'   are defined in USBdefs_std_dsc.h
'
   Symbol Out =         0
   Symbol In =          1
   Symbol PIC_EP_NUM_MASK = %01111000
   Symbol PIC_EP_DIR_MASK = %00000100
   Symbol EP00_OUT =   ($00 << 3)
   Symbol EP00_IN =    ($00 << 3) | 4
   Symbol EP01_OUT =   ($01 << 3)
   Symbol EP01_IN =    ($01 << 3) | 4
   Symbol EP02_OUT =   ($02 << 3)
   Symbol EP02_IN =    ($02 << 3) | 4
   Symbol EP03_OUT =   ($03 << 3)
   Symbol EP03_IN =    ($03 << 3) | 4
   Symbol EP04_OUT =   ($04 << 3)
   Symbol EP04_IN =    ($04 << 3) | 4
   Symbol EP05_OUT =   ($05 << 3)
   Symbol EP05_IN =    ($05 << 3) | 4
   Symbol EP06_OUT =   ($06 << 3)
   Symbol EP06_IN =    ($06 << 3) | 4
   Symbol EP07_OUT =   ($07 << 3)
   Symbol EP07_IN =    ($07 << 3) | 4
   Symbol EP08_OUT =   ($08 << 3)
   Symbol EP08_IN =    ($08 << 3) | 4
   Symbol EP09_OUT =   ($09 << 3)
   Symbol EP09_IN =    ($09 << 3) | 4
   Symbol EP10_OUT =   ($0A << 3)
   Symbol EP10_IN =    ($0A << 3) | 4
   Symbol EP11_OUT =   ($0B << 3)
   Symbol EP11_IN =    ($0B << 3) | 4
   Symbol EP12_OUT =   ($0C << 3)
   Symbol EP12_IN =    ($0C << 3) | 4
   Symbol EP13_OUT =   ($0D << 3)
   Symbol EP13_IN =    ($0D << 3) | 4
   Symbol EP14_OUT =   ($0E << 3)
   Symbol EP14_IN =    ($0E << 3) | 4
   Symbol EP15_OUT =   ($0F << 3)
   Symbol EP15_IN =    ($0F << 3) | 4

'USBdefs_std_dsc.h
   $define USB_DESCRIPTOR_DEVICE        $01
   $define USB_DESCRIPTOR_CONFIGURATION $02
   $define USB_DESCRIPTOR_STRING        $03
   $define USB_DESCRIPTOR_INTERFACE     $04
   $define USB_DESCRIPTOR_ENDPOINT      $05
'
' * USB Endpoint Definitions
' * This is used in the descriptors.
' * NOTE: Do not use these values for checking against USTAT.
' * To check against USTAT, use values defined in USBdrv.h
'
'USBdefs_std_dsc.h

   $define _EP_OUT   $00
   $define _EP_IN    $80

   $define _EP01_OUT $01
   $define _EP01_IN  $81
   $define _EP02_OUT $02
   $define _EP02_IN  $82
   $define _EP03_OUT $03
   $define _EP03_IN  $83
   $define _EP04_OUT $04
   $define _EP04_IN  $84
   $define _EP05_OUT $05
   $define _EP05_IN  $85
   $define _EP06_OUT $06
   $define _EP06_IN  $86
   $define _EP07_OUT $07
   $define _EP07_IN  $87
   $define _EP08_OUT $08
   $define _EP08_IN  $88
   $define _EP09_OUT $09
   $define _EP09_IN  $89
   $define _EP10_OUT $0A
   $define _EP10_IN  $8A
   $define _EP11_OUT $0B
   $define _EP11_IN  $8B
   $define _EP12_OUT $0C
   $define _EP12_IN  $8C
   $define _EP13_OUT $0D
   $define _EP13_IN  $8D
   $define _EP14_OUT $0E
   $define _EP14_IN  $8E
   $define _EP15_OUT $0F
   $define _EP15_IN  $8F

' configuration attributes
'USBdefs_std_dsc.h
   Symbol _DEFAULT =   $80              ' Default Value (Bit 7 is set)
   Symbol _SELF =      $40              ' Self-powered (Supports if set)
   Symbol _RWU =       $20              ' Remote Wakeup (Supports if set)

' endpoint transfer type
'USBdefs_std_dsc.h
   Symbol _CTRL =      $00              ' Control Transfer
   Symbol _ISO =       $01              ' Isochronous Transfer
   Symbol _BULK =      $02              ' Bulk Transfer
   Symbol _INTERRUPT = $03              ' Interrupt Transfer

'USBdefs_std_dsc.h
   Symbol _NS =        $00              ' No Synchronization
   Symbol _AS =        $04              ' Asynchronous
   Symbol _AD =        $08              ' Adaptive
   Symbol _SY =        $0C              ' Synchronous

' isochronous endpoint usage type
'USBdefs_std_dsc.h
   Symbol _DE =        $00              ' Data endpoint
   Symbol _FE =        $10              ' Feedback endpoint
   Symbol _IE =        $20              ' Implicit feedback Data endpoint

' Parameter definitions are defined in USBdrv.h
'USBcfg.h
   Symbol MODE_PP =  _PPBM0
   Symbol UCFG_VAL = _PUEN | _TRINT | _FS | MODE_PP
(*
 * MUID = Microchip USB Class ID
 * Used to identify which of the USB classes owns the current
 * session of control transfer over EP0
*)
'USBcfg.h
   Symbol MUID_NULL = 0
   Symbol MUID_USB9 = 1
   Symbol MUID_HID  = 2
   Symbol MUID_CDC  = 3
   Symbol MUID_MSD  = 4

' buffer descriptor STATUS register initialisation parameters
'USBmap.h
   Symbol _BSTALL =    $04                ' Buffer Stall enable
   Symbol _DTSEN =     $08                ' Data Toggle Synch enable
   Symbol _INCDIS =    $10                ' Address increment disable
   Symbol _KEN =       $20                ' SIE keeps buff descriptor s enable
   Symbol _DAT0 =      $00                ' DATA0 packet expected next
   Symbol _DAT1 =      $40                ' DATA1 packet expected next
   Symbol _DTSMASK =   $40                ' DTS Mask
   Symbol _USIE =      $80                ' SIE owns buffer
   Symbol _UCPU =      $00                ' CPU owns buffer

' USB device states - To be used with bUSB_Device_State
'USBmap.h
   Symbol DETACHED_STATE =    0
   Symbol ATTACHED_STATE =    1
   Symbol POWERED_STATE =     2
   Symbol DEFAULT_STATE =     3
   Symbol ADR_PENDING_STATE = 4
   Symbol ADDRESS_STATE =     5
   Symbol CONFIGURED_STATE =  6

' memory types for control transfer - used in Usb_Device_Status
$define _RAM_  0
$define _ROM_  1

' Standard Request Codes
' USB 2.0 Spec Ref Table 9-4
 'USB9.h
   Symbol GET_STATUS =  0
   Symbol CLR_FEATURE = 1
   Symbol SET_FEATURE = 3
   Symbol SET_ADR =     5
   Symbol GET_DSC =     6
   Symbol SET_DSC =     7
   Symbol GET_CFG =     8
   Symbol SET_CFG =     9
   Symbol GET_INTF =    10
   Symbol SET_INTF =    11
   Symbol SYNCH_FRAME = 12

' standard feature selectors
'USB9.h
   Symbol DEVICE_REMOTE_WAKEUP = $01
   Symbol ENDPOINT_HALT =        $00

' control transfer states
'USBctrltr.h
   Symbol WAIT_SETUP =  $00
   Symbol CTRL_TRF_TX = $01
   Symbol CTRL_TRF_RX = $02

' USB PID: Token Types - See chapter 8 in the USB specification
'USBctrltr.h
   Symbol SETUP_TOKEN = %00001101
   Symbol OUT_TOKEN =   %00000001
   Symbol IN_TOKEN =    %00001001

' bmRequestType definitions
'USBctrltr.h
   Symbol HOST_TO_DEV = $00
   Symbol DEV_TO_HOST = $01
   Symbol STANDARD =    $00
   Symbol CLASS =       $01
   Symbol VENDOR  =     $02
   Symbol RCPT_DEV =    $00
   Symbol RCPT_INTF =   $01
   Symbol RCPT_EP =     $02
   Symbol RCPT_OTH =    $03

' class-specific requests
'hid.h
   Symbol GET_REPORT =     $01
   Symbol GET_IDLE =       $02
   Symbol GET_PROTOCOL =   $03
   Symbol SET_report =     $09
   Symbol SET_IDLE =       $0A
   Symbol SET_PROTOCOL =   $0B

' class descriptor  types
'hid.h
   Symbol DSC_HID =        $21
   Symbol DSC_RPT =        $22
   Symbol DSC_PHY =        $23

' protocol selection
'hid.h
   Symbol BOOT_PROTOCOL =  $00
   Symbol RPT_PROTOCOL =   $01

' HID interface class code
'hid.h
   Symbol HID_INTF =       $03

' HID interface class subclass codes
'hid.h
   Symbol BOOT_INTF_SUBCLASS = $01

' HID interface class protocol codes
'hid.h
   Symbol HID_PROTOCOL_NONE =      $00
   Symbol HID_PROTOCOL_KEYBOAD =   $01
   Symbol HID_PROTOCOL_MOUSE =     $02

'cdc.h
   Symbol SEND_ENCAPSULATED_COMMAND =  $00
   Symbol GET_ENCAPSULATED_RESPONSE =  $01
   Symbol SET_COMM_FEATURE =           $02
   Symbol GET_COMM_FEATURE =           $03
   Symbol CLEAR_COMM_FEATURE =         $04
   Symbol SET_LINE_CODING =            $20
   Symbol GET_LINE_CODING =            $21
   Symbol SET_CONTROL_LINE_STATE =     $22
   Symbol SEND_BREAK =                 $23

' Note: Notifications are polled over
' Communication Interface (Interrupt Endpoint)
'cdc.h
   Symbol NETWORK_CONNECTION =         $00
   Symbol RESPONSE_AVAILABLE =         $01
   Symbol SERIAL_STATE =               $20

' Device Class Code
'cdc.h
   Symbol CDC_DEVICE =                 $02

' Communication Interface Class Code
'cdc.h
   Symbol COMM_INTF =                  $02

' Communication Interface Class SubClass Codes
'cdc.h
   Symbol ABSTRACT_CONTROL_MODEL =     $02

' Communication Interface Class Control Protocol Codes
'cdc.h
   Symbol V25TER =                     $01    ' Common AT commands ("Hayes(TM)")

' Data Interface Class Codes
'cdc.h
   Symbol DATA_INTF =                  $0A

' Data Interface Class Protocol Codes  'cdc.h
   Symbol NO_PROTOCOL =                $00    ' No class specific protocol required

' Communication Feature Selector  Codes
'cdc.h
   Symbol ABSTRACT_STATE =             $01
   Symbol COUNTRY_SETTING =            $02

' Functional descriptor s
' Type Values for the bDscType Field
'cdc.h
   Symbol CS_INTERFACE =               $24
   Symbol CS_ENDPOINT =                $25

' bDscSubType in Functional descriptor s
'cdc.h
   Symbol DSC_FN_HEADER =              $00
   Symbol DSC_FN_CALL_MGT =            $01
   Symbol DSC_FN_ACM =                 $02    ' ACM - Abstract Control Management
   Symbol DSC_FN_DLM =                 $03    ' DLM - Direct Line Managment
   Symbol DSC_FN_TELEPHONE_RINGER =    $04
   Symbol DSC_FN_RPT_CAPABILITIES =    $05
   Symbol DSC_FN_UNION =               $06
   Symbol DSC_FN_COUNTRY_SELECTION =   $07
   Symbol DSC_FN_TEL_OP_MODES =        $08
   Symbol DSC_FN_USB_TERMINAL =        $09
   ' more.... see Table 25 in USB CDC Specification 1.1

' CDC Bulk IN transfer states
'cdc.h
   Symbol CDC_TX_READY =               $00
   Symbol CDC_TX_BUSY =                $01
   Symbol CDC_TX_BUSY_ZLP =            $02    ' ZLP: Zero Length Packet
   Symbol CDC_TX_COMPLETING =          $03

' Device status bits

   Symbol RemoteWakeup = 0                    ' [0]Disabled [1]Enabled: See USBdrv.c,USB9.c
   Symbol ctrl_trf_mem = 1                    ' [0]RAM      [1]ROM

' Endpoint status bits

   Symbol BC8 = 0
   Symbol BC9 = 1
   Symbol BSTALL = 2       ' Buffer Stall Enable
   Symbol DTSEN = 3        ' Data Toggle Synch Enable
   Symbol INCDIS = 4       ' Address Increment Disable
   Symbol KEN = 5          ' BD Keep Enable
   Symbol DTS = 6          ' Data Toggle Synch Value
   Symbol UOWN = 7         ' USB Ownership

   Symbol PID0 = 2
   Symbol PID1 = 3
   Symbol PID2 = 4
   Symbol PID3 = 5
'
' Fuse Configuration Blocks for particular PICmicro devices
'
$if _device = _18F4550 Or _device = _18F2550
Config_Start
    PLLDIV = 3
    CPUDIV = OSC1_PLL2
    USBDIV = 1
    FOSC = HSPLL_HS
    IESO = Off
    PWRT = On
    BOR = On
    VREGEN = On
    WDT = Off
    WDTPS = 128
    PBADEN = Off
    STVREN = On
    LVP = Off
    ICPRT = Off
    XINST = Off
    Debug = Off
Config_End
$endif

$if _device = _18F87J50 Or _device = _18F86J50 Or _device = _18F85J50 Or _device = _18F67J50 Or _device = _18F66J50 Or _device = _18F65J50
Config_Start
    Debug = Off         ' Background debugger disabled and RB6 and RB7 configured as general purpose I/O pins
    XINST = Off         ' Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
    STVREN = Off        ' Reset on stack overflow/underflow disabled
    PLLDIV = 3		    ' Divide by 3 (12 MHz oscillator input)
    WDTEN = Off         ' WDT disabled (control is placed on SWDTEN bit)
    CP0 = Off           ' Program memory is not code-protected
    FCMEN = Off         ' Fail-Safe Clock Monitor disabled
    CPUDIV = OSC1		' No CPU System clock divide
    FOSC = HSPLL        ' HS oscillator, PLL enabled and under software control
    IESO = Off          ' Two-Speed Start-up disabled
    WDTPS = 128         ' 1:128
    Wait = Off          ' Wait states for operations on external memory bus disabled
    BW = 8              ' 8-bit external bus mode
    EASHFT = Off        ' Address shifting disabled, address on external bus reflects the PC value
    MODE = MM           ' Microcontroller mode - External bus disabled
    MSSPMSK = MSK7      ' 7 Bit address masking
    PMPMX = ALTERNATE   ' MP port pins not connected to EMB
    ECCPMX = DEFAULT    ' ECCP1 outputs (P1B/P1C) are multiplexed with RE6 and RE5
    CCP2MX = DEFAULT    ' CCP2 is multiplexed with RC1
Config_End
$endif

$if _device = _18F13K50 Or _device = _18F14K50
Config_Start
    CPUDIV = NOCLKDIV    ' No CPU System Clock divide
    USBDIV = Off         ' USB Clock comes directly from the OSC1/OSC2 oscillator block' no divide
    FOSC = HS            ' HS oscillator
    PLLEN = On           ' Oscillator multiplied by 4
    PCLKEN = On          ' Primary clock enabled
    FCMEN = Off          ' Fail-Safe Clock Monitor disabled
    IESO = Off           ' Oscillator Switchover mode disabled
    PWRTEN = On          ' PWRT enabled
    BOREN = On           ' Brown-out Reset enabled and controlled by software (SBOREN is enabled)
    BORV = 19            ' VBOR set to 1.9 V nominal
    WDTEN = Off          ' WDT is controlled by SWDTEN bit of the WDTCON register
    WDTPS = 128          ' Watchdog Timer Postscale 1:128
    MCLRE = On           ' MCLR pin enabled, RE3 input pin disabled
    HFOFST = Off         ' The system clock is held off until the HFINTOSC is stable.
    STVREN = Off         ' Stack full/underflow will not cause Reset
    LVP = Off            ' Single-Supply ICSP disabled
    BBSIZ = Off          ' 512W boot block size
    XINST = Off          ' Instruction set extension and Indexed Addressing mode disabled
    Debug = Off          ' Background debugger disabled and RA0 and RA1 configured as general purpose I/O pins
    CP0 = Off            ' Block 0 not code-protected
    CP1 = Off            ' Block 1 not code-protected
    CPB = Off            ' Boot block not code-protected
    CPD = Off            ' Data EEPROM not code-protected
    WRT0 = Off           ' Block 0 not write-protected
    WRT1 = Off           ' Block 1 not write-protected
    WRTB = Off           ' Boot block not write-protected
    WRTC = Off           ' Configuration registers not write-protected
    WRTD = Off           ' Data EEPROM not write-protected
    EBTR0 = Off          ' Block 0 not protected from table reads executed in other blocks
    EBTR1 = Off          ' Block 1 not protected from table reads executed in other blocks
    EBTRB = Off          ' Boot block not protected from table reads executed in other blocks
Config_End
$endif


