'
' USB HID Class routines
'
$ifdef USB_USE_HID

    Dim bHID_Rpt_Rx_Len As Byte System
(*
****************************************************************************
* Name    : mUSB_GetHIDDscAdr                                              *
* Purpose : Get HID descriptor address                                     *
****************************************************************************
*)
$define mUSB_GetHIDDscAdr()  '
    wSrc_Rom = VarPtr configDescriptor1  '
    wSrc_Rom = wSrc_Rom + 18

(*
****************************************************************************
* Name    : mUSB_GetHIDRptDscAdr                                           *
* Purpose : Get HID descriptor report address                              *
****************************************************************************
*)
$define mUSB_GetHIDRptDscAdr() '
   wSrc_Rom = VarPtr hid_rpt01
'
' Note: Subroutine "HID_HandleControlOutReport" should go here if required
'
(*
****************************************************************************
* Name    : HID_GetReportHandler                                           *
* Purpose : Called on receiving a Setup packet with a GET_REPORT request.  *
*         : Extracts the report type and the report ID of the report the   *
*         : device will send in the Data stage of the transfer. Calls a    *
*         : function to provide the data.	                               *
****************************************************************************
*)
$define HID_GetReportHandler() GoSub __HIDGetReportHandler

__HIDGetReportHandler:
'
' The report type is in the high byte of the setup packet's Value field.
' 3 = Feature; 1 = Input.
'
    Select SetupPkt_wValue.Byte1
        '
        ' Input report
        '
        Case 1
            ' Find out which report ID was specified.
            ' The report ID is in the low byte (LSB) of the Value field.
            '
            ' This example supports Report ID 0.
            '
            If SetupPkt_wValue.Byte0 = 0 Then       ' Report ID 0
                bCtrl_Trf_Session_Owner = MUID_HID  ' HID will handle the request.
                '
                ' Provide the data to send.
                '
                wSrc_Ram = VarPtr HID_Report_In
                USB_Count.Byte0 = HID_INPUT_REPORT_BYTES ' The number of bytes in the report
            'ElseIf SetupPkt_wValue.Byte0 =  1:     ' Report ID 1
                ' Place similar code to handle Report ID 1 here.
            EndIf
        '
        ' Feature report
        '
        Case 3
            ' The report ID is in the low byte of the Setup packet's Value field.
            ' This example supports Report ID 0.
            '
            If SetupPkt_wValue.Byte0 = 0 Then       ' Report ID 0
                bCtrl_Trf_Session_Owner = MUID_HID  ' HID will handle the request.
                '
                ' Provide the data to send.
                '
                wSrc_Ram = VarPtr HID_Report_Feature
                USB_Count.Byte0 = HID_FEATURE_REPORT_BYTES ' The number of bytes in the report
            EndIf
    EndSelect
    bUSB_Stat.ctrl_trf_mem = _RAM_                  ' Set memory type to RAM
    Return
(*
****************************************************************************
* Name    : HID_SetReportHandler                                           *
* Purpose : Called on receiving a Setup packet with a SET_REPORT request.  *
* 		  : Extracts the report type and the report ID of the report the   *
*         : host will send IN the Data stage of the transfer.              *
*         : Places the report type and report ID IN out_report_status.	   *
****************************************************************************
*)
$define HID_SetReportHandler() GoSub __HIDSetReportHandler

__HIDSetReportHandler:
'
' The report type is in the high byte of the Setup packet's Value field.
' 3 = Feature; 2 = Output.
'
    Select SetupPkt_wValue.Byte1
        Case 2                              ' Output report
            '
            ' The report ID is in the low byte of the Setup packet's Value field.
            ' This example supports Report ID 0.
            '
            If SetupPkt_wValue.Byte0 = 0 Then                       ' Report ID 0
                bCtrl_Trf_Session_Owner = MUID_HID                  ' HID will handle the request.
                '
                ' Save the report type and report ID in out_report_status.
                '
                ' out_report_status = SetOutReportStatus (MSB(SetupPkt_wValue), LSB(SetupPkt_wValue));
                '
                ' When the report arrives in the data stage, the data will be
                ' stored in HID_Report_Out (defined in USBmmap.c).
                '
                wDest_Ram = VarPtr HID_Report_Out
            EndIf
        Case 3                              ' Feature report
            '
            ' Find out which report ID was specified.
            ' The report ID is in the low byte (LSB) of the Value field.
            '
            ' This example supports Report ID 0.
            '
            If SetupPkt_wValue.Byte0 = 0 Then                       ' Report ID 0
                bCtrl_Trf_Session_Owner = MUID_HID                  ' HID will handle the request.
                '
                ' Save the report type and report ID in out_report_status.
                '
                ' out_report_status = SetOutReportStatus (MSB(SetupPkt_wValue), LSB(SetupPkt_wValue));
                '
                ' When the report arrives in the data stage, the data will be
                ' stored in HID_Report_Feature (defined in USBmmap.c).
                '
                wDest_Ram = VarPtr HID_Report_Feature
            EndIf
    EndSelect
    Return
(*
****************************************************************************
* Name    : USB_CheckHIDRequest                                            *
* Purpose : This routine checks the setup data packet to see if it knows   *
*         : how to handle it                                               *
****************************************************************************
*)
$define USB_CheckHIDRequest() GoSub __USBCheckHIDRequest

__USBCheckHIDRequest:
'
' The packet must be directed to an interface with the same ID as the device's HID interface.
'
    WREG = SetupPkt_Recipient_f5 & $1F              ' Get Recipient
    If WREG <> RCPT_INTF Then Return
    If SetupPkt_bIntfID <> HID_INTF_ID Then Return
'
' There are two standard requests that hid may support.
' 1. GET_DSC(DSC_HID, DSC_RPT, DSC_PHY)
' 2. SET_DSC(DSC_HID, DSC_RPT, DSC_PHY)
'
    If SetupPkt_bRequest = GET_DSC Then
        '
        ' Request for a descriptor.
        '
        Select SetupPkt_bDscType
            '
            ' HID descriptor.
            '
            Case DSC_HID
                bCtrl_Trf_Session_Owner = MUID_HID
                mUSB_GetHIDDscAdr()                 ' Get HID descriptor address
                USB_Count = 9                       ' Size of the "HID class-specific descriptor" section within the configDescriptor1 table
            '
            ' Report descriptor.
            '
            Case DSC_RPT
                bCtrl_Trf_Session_Owner = MUID_HID
                mUSB_GetHIDRptDscAdr()              ' Address of the hid_rpt01 table
                USB_Count = HID_RPT01_SIZE          ' Get HID report descriptor size
            ' Physical descriptor.
            'case DSC_PHY:
                'bCtrl_Trf_Session_Owner = MUID_HID
        EndSelect
        bUSB_Stat.ctrl_trf_mem = _ROM_
    EndIf
    WREG = SetupPkt_RequestType_f2 >> 5             ' \ Get Request Type
    WREG = WREG & %00000011                         ' /
    If WREG <> CLASS Then Return
'
' HID-specific requests.
'
    Select SetupPkt_bRequest
        '
        ' required for all HID tranfers
        '
        Case GET_REPORT
            HID_GetReportHandler()
        '
        ' optional - needed for control transfers only
        '
        Case SET_report
            HID_SetReportHandler()
        '
        ' optional
        '
        Case GET_IDLE
            bCtrl_Trf_Session_Owner = MUID_HID
            wSrc_Ram = VarPtr bUSB_Idle_Rate        ' Set source
            bUSB_Stat.ctrl_trf_mem = _RAM_          ' Set memory type
            USB_Count.Byte0 = 1                     ' Set data count
        '
        ' optional - except for keyboards using the boot protocol
        '
        Case SET_IDLE
            bCtrl_Trf_Session_Owner = MUID_HID
            bUSB_Idle_Rate = SetupPkt_wValue.Byte1
        '
        ' required for HIDs that support the boot protocol
        '
        Case GET_PROTOCOL
            bCtrl_Trf_Session_Owner = MUID_HID
            wSrc_Ram = VarPtr bUSB_ActiveProtocol   ' Set source
            bUSB_Stat.ctrl_trf_mem = _RAM_          ' Set memory type
            USB_Count.Byte0 = 1                     ' Set data count
        '
        ' required for HIDs that support the boot protocol
        '
        Case SET_PROTOCOL
            bCtrl_Trf_Session_Owner = MUID_HID
            bUSB_ActiveProtocol = SetupPkt_wValue.Byte0
    EndSelect
  Return
(*
****************************************************************************
* Name    : HID_InitEP                                                     *
* Purpose : HID_InitEP initialises HID endpoints, buffer descriptors,      *
*         : internal state-machine, and variables. It should be called     *
*         : after the USB host has sent out a SET_CONFIGURATION request.   *
****************************************************************************
*)
$define HID_InitEP() GoSub __HIDInitEP

__HIDInitEP:
    UEP1 = EP_OUT_IN | HSHK_EN                  ' Enable 2 data pipes
    HID_BD_OUT_Cnt = HID_BUFFER_SIZE            ' Set buffer size
    HID_BD_OUT_Adr = VarPtr HID_Report_Out      ' Set buffer address
    HID_BD_OUT_Stat = _USIE | _DAT0 | _DTSEN    ' Set status
(*
    Do not have to init Cnt of in pipes here.
    Reason: Number of bytes to send to the host varies from one transaction to another.
            Cnt should equal the exact number of bytes to transmit for a given in transaction.
            This number of bytes will only be known right before the data is sent.
*)
    HID_BD_IN_Adr = VarPtr HID_Report_In        ' Set buffer address
    HID_BD_IN_Stat = _UCPU | _DAT1              ' Set status
    Return
'
' End of HID Class
$endif




