'
' USB Memory Allocation
'
    Declare Onboard_USB = Off               ' Disable the compiler's USB RAM configuration
    Declare Available_RAM = _USB_ram        ' Adjust the amount of RAM the device contains
    #disable USB_ALL                        ' Disable the compiler's USB routines

    Include "USB_defs.inc"                   ' Load the USB Definitions into the program
'
' Hardware Register Definitions
'
    Dim wUSB_FSR0 As FSR0L.Word
    Dim wUSB_FSR1 As FSR1L.Word
    Dim wUSB_TBLPTR As TBLPTRL.Word
'
' Memory Allocation for the USB stack
'
    Dim bUSB_Flags              As Byte System
    Dim tUSB_Attached           As bUSB_Flags.0      ' 1 if the USB bus is enumerated
    Dim bUSB_Device_State       As Byte System
    Dim bUSB_Length             As Byte System
    Dim bUSB_Idle_Rate          As Byte System
    Dim bUSB_ActiveProtocol     As Byte System
    Dim wUSB_BytesToSend        As Word System
    Dim wUSB_Temp               As wUSB_BytesToSend
    Dim wUSB_BytesToRead        As Word System
    Dim bUSB_Stat               As Byte System
    Dim bUSB_Active_Cfg         As Byte System
    Dim bCtrl_Trf_State         As Byte System
    Dim bCtrl_Trf_Session_Owner As Byte System
    Dim wSrc_Rom                As Word System
    Dim wSrc_Ram                As wSrc_Rom
    Dim wDest_Ram               As Word System
    Dim USB_Count               As Byte System

    'Dim bUSB_alt_intf[MAX_NUM_INT] As Byte
    Dim bUSB_alt_intf           As Byte System
'
' Memory Allocation for USB Dual Port RAM
'
$define cEP_SIZE 4                      ' endpoint size (in bytes)
$define cUSB_RAM_EP _USB_ram			' ** Start of USB Dual-Port RAM **
'
' Endpoint 0, OUT and IN
$if (MAX_EP_NUMBER >= 0)
    Dim ep0Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 0)
    Dim ep0Bo_Stat_PID As ep0Bo.Byte0
    Dim ep0Bo_Stat As ep0Bo.Byte0
    Dim ep0Bo_Cnt As ep0Bo.Byte1
    Dim ep0Bo_Adr As ep0Bo.Word1
    Dim ep0Bo_AdrL As ep0Bo.Byte2

    Dim ep0Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 1)
    Dim ep0Bi_Stat_PID As ep0Bi.Byte0
    Dim ep0Bi_Stat As ep0Bi.Byte0
    Dim ep0Bi_Cnt As ep0Bi.Byte1
    Dim ep0Bi_Adr As ep0Bi.Word1
    Dim ep0Bi_AdrL As ep0Bi.Byte3
$endif
'
' Endpoint 1, OUT and IN
$if (MAX_EP_NUMBER >= 1)
    Dim ep1Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 2)
    Dim ep1Bo_Stat_PID As ep1Bo.Byte0
    Dim ep1Bo_Stat As ep1Bo.Byte0
    Dim ep1Bo_Cnt As ep1Bo.Byte1
    Dim ep1Bo_Adr As ep1Bo.Word1
    Dim ep1Bo_AdrL As ep1Bo.Byte2

    Dim ep1Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 3)
    Dim ep1Bi_Stat_PID As ep1Bi.Byte0
    Dim ep1Bi_Stat As ep1Bi.Byte0
    Dim ep1Bi_Cnt As ep1Bi.Byte1
    Dim ep1Bi_Adr As ep1Bi.Word1
    Dim ep1Bi_AdrL As ep1Bi.Byte2
$endif
'
' Endpoint 2, OUT and IN
$if (MAX_EP_NUMBER >= 2)
    Dim ep2Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 4)
    Dim ep2Bo_Stat_PID As ep2Bo.Byte0
    Dim ep2Bo_Stat As ep2Bo.Byte0
    Dim ep2Bo_Cnt As ep2Bo.Byte1
    Dim ep2Bo_Adr As ep2Bo.Word1
    Dim ep2Bo_AdrL As ep2Bo.Byte2

    Dim ep2Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 5)
    Dim ep2Bi_Stat_PID As ep2Bi.Byte0
    Dim ep2Bi_Stat As ep2Bi.Byte0
    Dim ep2Bi_Cnt As ep2Bi.Byte1
    Dim ep2Bi_Adr As ep2Bi.Word1
    Dim ep2Bi_AdrL As ep2Bi.Byte2
$endif
'
' Endpoint 3, OUT and IN
$if (MAX_EP_NUMBER >= 3)
    Dim ep3Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 6)
    Dim ep3Bo_Stat_PID As ep3Bo.Byte0
    Dim ep3Bo_Stat As ep3Bo.Byte0
    Dim ep3Bo_Cnt As ep3Bo.Byte1
    Dim ep3Bo_Adr As ep3Bo.Word1
    Dim ep3Bo_AdrL As ep3Bo.Byte2

    Dim ep3Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 7)
    Dim ep3Bi_Stat_PID As ep3Bi.Byte0
    Dim ep3Bi_Stat As ep3Bi.Byte0
    Dim ep3Bi_Cnt As ep3Bi.Byte1
    Dim ep3Bi_Adr As ep3Bi.Word1
    Dim ep3Bi_AdrL As ep3Bi.Byte2
$endif
'
' Endpoint 4, OUT and IN
$if (MAX_EP_NUMBER >= 4)
    Dim ep4Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 8)
    Dim ep4Bo_Stat_PID As ep4Bo.Byte0
    Dim ep4Bo_Stat As ep4Bo.Byte0
    Dim ep4Bo_Cnt As ep4Bo.Byte1
    Dim ep4Bo_Adr As ep4Bo.Word1
    Dim ep4Bo_AdrL As ep4Bo.Byte2

    Dim ep4Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 9)
    Dim ep4Bi_Stat_PID As ep4Bi.Byte0
    Dim ep4Bi_Stat As ep4Bi.Byte0
    Dim ep4Bi_Cnt As ep4Bi.Byte1
    Dim ep4Bi_Adr As ep4Bi.Word1
    Dim ep4Bi_AdrL As ep4Bi.Byte2
$endif
'
' Endpoint 5, OUT and IN
$if (MAX_EP_NUMBER >= 5)
    Dim ep5Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 10)
    Dim ep5Bo_Stat_PID As ep5Bo.Byte0
    Dim ep5Bo_Stat As ep5Bo.Byte0
    Dim ep5Bo_Cnt As ep5Bo.Byte1
    Dim ep5Bo_Adr As ep5Bo.Word1
    Dim ep5Bo_AdrL As ep5Bo.Byte2

    Dim ep5Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 11)
    Dim ep5Bi_Stat_PID As ep5Bi.Byte0
    Dim ep5Bi_Stat As ep5Bi.Byte0
    Dim ep5Bi_Cnt As ep5Bi.Byte1
    Dim ep5Bi_Adr As ep5Bi.Word1
    Dim ep5Bi_AdrL As ep5Bi.Byte2
$endif
'
' Endpoint 6, OUT and IN
$if (MAX_EP_NUMBER >= 6)
    Dim ep6Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 12)
    Dim ep6Bo_Stat_PID As ep6Bo.Byte0
    Dim ep6Bo_Stat As ep6Bo.Byte0
    Dim ep6Bo_Cnt As ep6Bo.Byte1
    Dim ep6Bo_Adr As ep6Bo.Word1
    Dim ep6Bo_AdrL As ep6Bo.Byte2

    Dim ep6Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 13)
    Dim ep6Bi_Stat_PID As ep6Bi.Byte0
    Dim ep6Bi_Stat As ep6Bi.Byte0
    Dim ep6Bi_Cnt As ep6Bi.Byte1
    Dim ep6Bi_Adr As ep6Bi.Word1
    Dim ep6Bi_AdrL As ep6Bi.Byte2
$endif
'
' Endpoint 7, OUT and IN
$if (MAX_EP_NUMBER >= 7)
    Dim ep7Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 14)
    Dim ep7Bo_Stat_PID As ep7Bo.Byte0
    Dim ep7Bo_Stat As ep7Bo.Byte0
    Dim ep7Bo_Cnt As ep7Bo.Byte1
    Dim ep7Bo_Adr As ep7Bo.Word1
    Dim ep7Bo_AdrL As ep7Bo.Byte2

    Dim ep7Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 15)
    Dim ep7Bi_Stat_PID As ep7Bi.Byte0
    Dim ep7Bi_Stat As ep7Bi.Byte0
    Dim ep7Bi_Cnt As ep7Bi.Byte1
    Dim ep7Bi_Adr As ep7Bi.Word1
    Dim ep7Bi_AdrL As ep7Bi.Byte2
$endif
'
' Endpoint 8, OUT and IN
$if (MAX_EP_NUMBER >= 8)
    Dim ep8Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 16)
    Dim ep8Bo_Stat_PID As ep8Bo.Byte0
    Dim ep8Bo_Stat As ep8Bo.Byte0
    Dim ep8Bo_Cnt As ep8Bo.Byte1
    Dim ep8Bo_Adr As ep8Bo.Word1
    Dim ep8Bo_AdrL As ep8Bo.Byte2

    Dim ep8Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 17)
    Dim ep8Bi_Stat_PID As ep8Bi.Byte0
    Dim ep8Bi_Stat As ep8Bi.Byte0
    Dim ep8Bi_Cnt As ep8Bi.Byte1
    Dim ep8Bi_Adr As ep8Bi.Word1
    Dim ep8Bi_AdrL As ep8Bi.Byte2
$endif
'
' Endpoint 9, OUT and IN
$if (MAX_EP_NUMBER >= 9)
    Dim ep9Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 18)
    Dim ep9Bo_Stat_PID As ep9Bo.Byte0
    Dim ep9Bo_Stat As ep9Bo.Byte0
    Dim ep9Bo_Cnt As ep9Bo.Byte1
    Dim ep9Bo_Adr As ep9Bo.Word1
    Dim ep9Bo_AdrL As ep9Bo.Byte2

    Dim ep9Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 19)
    Dim ep9Bi_Stat_PID As ep9Bi.Byte0
    Dim ep9Bi_Stat As ep9Bi.Byte0
    Dim ep9Bi_Cnt As ep9Bi.Byte1
    Dim ep9Bi_Adr As ep9Bi.Word1
    Dim ep9Bi_AdrL As ep9Bi.Byte2
$endif
'
' Endpoint 10, OUT and IN
$if (MAX_EP_NUMBER >= 10)
    Dim ep10Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 20)
    Dim ep10Bo_Stat_PID As ep10Bo.Byte0
    Dim ep10Bo_Stat As ep10Bo.Byte0
    Dim ep10Bo_Cnt As ep10Bo.Byte1
    Dim ep10Bo_Adr As ep10Bo.Word1
    Dim ep10Bo_AdrL As ep10Bo.Byte2

    Dim ep10Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 21)
    Dim ep10Bi_Stat_PID As ep10Bi.Byte0
    Dim ep10Bi_Stat As ep10Bi.Byte0
    Dim ep10Bi_Cnt As ep10Bi.Byte1
    Dim ep10Bi_Adr As ep10Bi.Word1
    Dim ep10Bi_AdrL As ep10Bi.Byte2
$endif
'
' Endpoint 11, OUT and IN
$if (MAX_EP_NUMBER >= 11)
    Dim ep11Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 22)
    Dim ep11Bo_Stat_PID As ep11Bo.Byte0
    Dim ep11Bo_Stat As ep11Bo.Byte0
    Dim ep11Bo_Cnt As ep11Bo.Byte1
    Dim ep11Bo_Adr As ep11Bo.Word1
    Dim ep11Bo_AdrL As ep11Bo.Byte2

    Dim ep11Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 23)
    Dim ep11Bi_Stat_PID As ep11Bi.Byte0
    Dim ep11Bi_Stat As ep11Bi.Byte0
    Dim ep11Bi_Cnt As ep11Bi.Byte1
    Dim ep11Bi_Adr As ep11Bi.Word1
    Dim ep11Bi_AdrL As ep11Bi.Byte2
$endif
'
' Endpoint 12, OUT and IN
$if (MAX_EP_NUMBER >= 12)
    Dim ep12Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 24)
    Dim ep12Bo_Stat_PID As ep12Bo.Byte0
    Dim ep12Bo_Stat As ep12Bo.Byte0
    Dim ep12Bo_Cnt As ep12Bo.Byte1
    Dim ep12Bo_Adr As ep12Bo.Word1
    Dim ep12Bo_AdrL As ep12Bo.Byte2

    Dim ep12Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 25)
    Dim ep12Bi_Stat_PID As ep12Bi.Byte0
    Dim ep12Bi_Stat As ep12Bi.Byte0
    Dim ep12Bi_Cnt As ep12Bi.Byte1
    Dim ep12Bi_Adr As ep12Bi.Word1
    Dim ep12Bi_AdrL As ep12Bi.Byte2
$endif
'
' Endpoint 13, OUT and IN
$if (MAX_EP_NUMBER >= 13)
    Dim ep13Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 26)
    Dim ep13Bo_Stat_PID As ep13Bo.Byte0
    Dim ep13Bo_Stat As ep13Bo.Byte0
    Dim ep13Bo_Cnt As ep13Bo.Byte1
    Dim ep13Bo_Adr As ep13Bo.Word1
    Dim ep13Bo_AdrL As ep13Bo.Byte2

    Dim ep13Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 27)
    Dim ep13Bi_Stat_PID As ep13Bi.Byte0
    Dim ep13Bi_Stat As ep13Bi.Byte0
    Dim ep13Bi_Cnt As ep13Bi.Byte1
    Dim ep13Bi_Adr As ep13Bi.Word1
    Dim ep13Bi_AdrL As ep13Bi.Byte2
$endif
'
' Endpoint 14, OUT and IN
$if (MAX_EP_NUMBER >= 14)
    Dim ep14Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 28)
    Dim ep14Bo_Stat_PID As ep14Bo.Byte0
    Dim ep14Bo_Stat As ep14Bo.Byte0
    Dim ep14Bo_Cnt As ep14Bo.Byte1
    Dim ep14Bo_Adr As ep14Bo.Word1
    Dim ep14Bo_AdrL As ep14Bo.Byte2

    Dim ep14Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 29)
    Dim ep14Bi_Stat_PID As ep14Bi.Byte0
    Dim ep14Bi_Stat As ep14Bi.Byte0
    Dim ep14Bi_Cnt As ep14Bi.Byte1
    Dim ep14Bi_Adr As ep14Bi.Word1
    Dim ep14Bi_AdrL As ep14Bi.Byte2
$endif
'
' endpoint 15, OUT and IN
$if (MAX_EP_NUMBER >= 15)
    Dim ep15Bo As Dword At (cUSB_RAM_EP + cEP_SIZE * 30)
    Dim ep15Bo_Stat_PID As ep15Bo.Byte0
    Dim ep15Bo_Stat As ep15Bo.Byte0
    Dim ep15Bo_Cnt As ep15Bo.Byte1
    Dim ep15Bo_Adr As ep15Bo.Word1
    Dim ep15Bo_AdrL As ep15Bo.Byte2

    Dim ep15Bi As Dword At (cUSB_RAM_EP + cEP_SIZE * 31)
    Dim ep15Bi_Stat_PID As ep15Bi.Byte0
    Dim ep15Bi_Stat As ep15Bi.Byte0
    Dim ep15Bi_Cnt As ep15Bi.Byte1
    Dim ep15Bi_Adr As ep15Bi.Word1
    Dim ep15Bi_AdrL As ep15Bi.Byte2
$endif
'
' Located after endpoints...
'
    Symbol USB_RAM_BUFF = cUSB_RAM_EP + cEP_SIZE * (MAX_EP_NUMBER + 1) * 2

    Dim SetupPkt[USB_EP0_BUFF_SIZE] As Byte At USB_RAM_BUFF
'
' Create the equivalent of a structure
'
    Dim SetupPkt_bmRequestType As SetupPkt#0
    Dim SetupPkt_bRequest As SetupPkt#1

    Dim dSetupPkt_wValue As SetupPkt#2              ' \ Create the low and high parts of a word variable
    Dim dSetupPkt_wValueH As SetupPkt#3             ' /
    Dim SetupPkt_wValue As dSetupPkt_wValue.Word    ' Create a new word variable aliased to the 2 new parts
    Dim dSetupPkt_wIndex As SetupPkt#4              ' \ Create the low and high parts of a word variable
    Dim dSetupPkt_wIndexH As SetupPkt#5             ' /
    Dim SetupPkt_wIndex As dSetupPkt_wIndex.Word    ' Create a new word variable aliased to the 2 new parts
    Dim dSetupPkt_wLength As SetupPkt#6             ' \ Create the low and high parts of a word variable
    Dim dSetupPkt_wLengthH As SetupPkt#7            ' /
    Dim SetupPkt_wLength As dSetupPkt_wLength.Word  ' Create a new word variable aliased to the 2 new parts

    Dim SetupPkt_Recipient_f5 As SetupPkt#0    ' Device,Interface,Endpoint,Other (0..4)
    Dim SetupPkt_RequestType_f2 As SetupPkt#0  ' Standard,Class,Vendor,Reserved  (5..6)
    Dim SetupPkt_DataDir As SetupPkt#0.7       ' Host-to-device,Device-to-host
    Dim SetupPkt_bFeature As SetupPkt#2        ' DEVICE_REMOTE_WAKEUP, ENDPOINT_HALT

    Dim SetupPkt_bDscIndex As SetupPkt#2       ' For Configuration and String DSC Only
    Dim SetupPkt_bDscType As SetupPkt#3        ' Device,Configuration,String

    Dim SetupPkt_wLangID As SetupPkt_wIndex    ' Language ID

    Dim SetupPkt_bDevADR As SetupPkt#2         ' Device Address 0-127
    Dim SetupPkt_bDevADRH As SetupPkt#3        ' Must equal zero

    Dim SetupPkt_bCfgValue As SetupPkt#2       ' Configuration Value 0-255
    Dim SetupPkt_bCfgRSD As SetupPkt#3         ' Must equal zero (Reserved)

    Dim SetupPkt_bAltID As SetupPkt#2          ' Alternate Setting Value 0-255
    Dim SetupPkt_bAltIDH As SetupPkt#3         ' Must equal zero
    Dim SetupPkt_bIntfID As SetupPkt#4         ' Interface Number Value 0-255
    Dim SetupPkt_bIntfIDH As SetupPkt#5        ' Must equal zero

    Dim SetupPkt_bEPID As SetupPkt#4           ' Endpoint ID (Number & Direction)
    Dim SetupPkt_bEPIDH As SetupPkt#5          ' Must equal zero

    Dim SetupPkt_EPNum_f4 As SetupPkt#4        ' Endpoint Number 0-15 (0..3)
    Dim SetupPkt_EPDir As SetupPkt#4.7         ' Endpoint Direction: 0-OUT, 1-IN

    Dim bCtrlTrfData[USB_EP0_BUFF_SIZE] As Byte At (USB_RAM_BUFF + USB_EP0_BUFF_SIZE)
'
' Create the equivalent of a structure
'
    Dim CtrlTrfData_Byte0 As bCtrlTrfData#0
    Dim CtrlTrfData_Byte1 As bCtrlTrfData#1
    Dim CtrlTrfData_bmRequestType As bCtrlTrfData#0
    Dim CtrlTrfData_bRequest As bCtrlTrfData#1

    Dim dCtrlTrfData_wValue As bCtrlTrfData#2               ' \ Create the low and high parts of a word variable
    Dim dCtrlTrfData_wValueH As bCtrlTrfData#3              ' /
    Dim CtrlTrfData_wValue As dCtrlTrfData_wValue.Word      ' Create a new word variable aliased to the 2 new parts
    Dim dCtrlTrfData_wIndex As bCtrlTrfData#4               ' \ Create the low and high parts of a word variable
    Dim dCtrlTrfData_wIndexH As bCtrlTrfData#5              ' /
    Dim CtrlTrfData_wIndex As dCtrlTrfData_wIndex.Word      ' Create a new word variable aliased to the 2 new parts
    Dim dCtrlTrfData_wLength As bCtrlTrfData#6              ' \ Create the low and high parts of a word variable
    Dim dCtrlTrfData_wLengthH As bCtrlTrfData#7             ' /
    Dim CtrlTrfData_wLength As dCtrlTrfData_wLength.Word    ' Create a new word variable aliased to the 2 new parts

    Dim CtrlTrfData_Recipient_f5 As bCtrlTrfData#0    ' Device,Interface,Endpoint,Other (0..4)
    Dim CtrlTrfData_RequestType_f2 As bCtrlTrfData#0  ' Standard,Class,Vendor,Reserved  (5..6)
    Dim CtrlTrfData_DataDir As bCtrlTrfData#0.7       ' Host-to-device,Device-to-host
    Dim CtrlTrfData_bFeature As bCtrlTrfData#2        ' DEVICE_REMOTE_WAKEUP, ENDPOINT_HALT

    Dim CtrlTrfData_bDscIndex As bCtrlTrfData#2       ' For Configuration and String DSC Only
    Dim CtrlTrfData_bDscType As bCtrlTrfData#3        ' Device,Configuration,String

    Dim CtrlTrfData_wLangID As CtrlTrfData_wIndex     ' Language ID

    Dim CtrlTrfData_bDevADR As bCtrlTrfData#2         ' Device Address 0-127
    Dim CtrlTrfData_bDevADRH As bCtrlTrfData#3        ' Must equal zero

    Dim CtrlTrfData_bCfgValue As bCtrlTrfData#2       ' Configuration Value 0-255
    Dim CtrlTrfData_bCfgRSD As bCtrlTrfData#3         ' Must equal zero (Reserved)

    Dim CtrlTrfData_bAltID As bCtrlTrfData#2          ' Alternate Setting Value 0-255
    Dim CtrlTrfData_bAltIDH As bCtrlTrfData#3         ' Must equal zero
    Dim CtrlTrfData_bIntfID As bCtrlTrfData#4         ' Interface Number Value 0-255
    Dim CtrlTrfData_bIntfIDH As bCtrlTrfData#5        ' Must equal zero

    Dim CtrlTrfData_bEPID As bCtrlTrfData#4           ' Endpoint ID (Number & Direction)
    Dim CtrlTrfData_bEPIDH As bCtrlTrfData#5          ' Must equal zero

    Dim CtrlTrfData_EPNum_f4 As bCtrlTrfData#4        ' Endpoint Number 0-15 (0..3)
    Dim CtrlTrfData_EPDir As bCtrlTrfData#4.7         ' Endpoint Direction: 0-OUT, 1-IN
'
' HID - located after endpoint buffers...
$ifdef USB_USE_HID
    Symbol tHID_RxIsBusy = ep1Bo_Stat.UOWN
    Symbol tHID_TxIsBusy = ep1Bi_Stat.UOWN

    Symbol USB_VAR_HID = (USB_RAM_BUFF + (USB_EP0_BUFF_SIZE * 2))

    Dim HID_Report_Out[HID_INT_OUT_EP_SIZE] As Byte At USB_VAR_HID
    Dim HID_Report_In[HID_INT_IN_EP_SIZE] As Byte At (USB_VAR_HID + HID_INT_OUT_EP_SIZE)
    Dim HID_Report_Feature[USB_EP0_BUFF_SIZE] As Byte At (USB_VAR_HID + HID_INT_OUT_EP_SIZE + HID_INT_IN_EP_SIZE)

    Dim HID_UEP As UEP1
    Dim HID_BD_OUT As ep1Bo
    Dim HID_BD_OUT_Stat_PID As ep1Bo.Byte0
    Dim HID_BD_OUT_Stat As ep1Bo.Byte0
    Dim HID_BD_OUT_Cnt As ep1Bo.Byte1
    Dim HID_BD_OUT_Adr As ep1Bo.Word1
    Dim HID_BD_OUT_AdrL As ep1Bo.Byte2

    Dim HID_BD_IN As ep1Bi
    Dim HID_BD_IN_Stat_PID As ep1Bi.Byte0
    Dim HID_BD_IN_Stat As ep1Bi.Byte0
    Dim HID_BD_IN_Cnt As ep1Bi.Byte1
    Dim HID_BD_IN_Adr As ep1Bi.Word1
    Dim HID_BD_IN_AdrL As ep1Bi.Byte2

$endif ' USB_USE_HID
'
' Global buffers - these are mapped onto USB RAM space (when possible), thus saving program RAM.
'
$ifdef USB_EXTENDED_RAM
'
' You can use a large 256 byte buffer, which can be useful for transferring large blocks of data.
' The RX and TX buffers match the HID report size.
' Note that these buffers are only available for USB devices that have extended dual port USB RAM
'
    $define _BufferRAM $0600
    $define _TXReportRAM _BufferRAM
    $define _RXReportRAM _BufferRAM + HID_REPORT_BYTES_IN
    Dim TXReport As String * (HID_REPORT_BYTES_IN - 1) At _TXReportRAM
    Dim RXReport As String * (HID_REPORT_BYTES_OUT - 1) At _RXReportRAM
$else ' USB Extended Ram

    $if HID_BUFFER_SIZE <= 64   ' We can use dual port RAM if the buffer is less than or equal 64 bytes
        $define _BufferRAM HID_Report_Feature + USB_EP0_BUFF_SIZE
        $define _TXReportRAM _BufferRAM
        $define _RXReportRAM _BufferRAM + HID_REPORT_BYTES_IN

        Dim TXReport As String * (HID_REPORT_BYTES_IN - 1) At _TXReportRAM
        Dim RXReport As String * (HID_REPORT_BYTES_OUT - 1) At _RXReportRAM
    $else ' HID buffer size
        Dim TXReport As String * (HID_REPORT_BYTES_IN - 1)
        Dim RXReport As String * (HID_REPORT_BYTES_OUT - 1)
    $endif

    $if (HID_BUFFER_SIZE >= 2)
        Dim TXReportH As TXReport#1
    $endif

    $if (HID_BUFFER_SIZE >= 4)
        Dim TXReportHH As TXReport#2
        Dim TXReportHHH As TXReport#3
    $endif
$endif


