
    Dim USB_wTimeToConnect As Word

    GoTo OverUSBSubs                           ' Jump over the subroutines

$define mGetEPNum() SetupPkt_EPNum_f4 & $0F
$define mGetRecipient() SetupPkt_Recipient_f5 & $1F

(*
****************************************************************************
* Name    : USB_InitialiseUSBDriver                                        *
* Purpose : Configures the USB module                                      *
*         : This determines: USB Speed, On-chip pull-up resistor           *
*         : selection, On-chip tranceiver selection, bus eye pattern       *
*         : generation mode, Ping-pong buffering mode selection            *
****************************************************************************
*)
$define USB_InitialiseUSBDriver() GoSub __mInitialiseUSBDriver

__mInitialiseUSBDriver:
'
' Make sure the USB routines can access code memory for the descriptor tables
'
$ifdef _EECON1
    $ifdef EECON1bits_EEPGD
        EECON1 = 0
        EECON1bits_EEPGD = 1
    $endif
$endif

$ifdef OSCTUNEbits_PLLEN
    OSCTUNEbits_PLLEN = 1
$endif

    tUSB_Attached = False
    USB_wTimeToConnect = 0
'
' HID defaults...
'
$ifdef USB_USE_HID
    bUSB_Idle_Rate = 0
    bUSB_ActiveProtocol = 0
$endif

$ifdef USB_SERVICE                  ' Are we using the low priority interrupt?
    IPR2bits_USBIP = 0              ' Yes. So USB Interrupt Priority set to Low
$endif
    UCFG = UCFG_VAL
    bUSB_Device_State = DETACHED_STATE
    bUSB_Stat = 0
    bUSB_Active_Cfg = 0
    Return
(*
****************************************************************************
* Name    : mDisableEP1toEPn                                               *
* Purpose : disable all endpoints except EP0. This routine                 *
*         : should be called when the host sends a RESET signal or a       *
*         : SET_CONFIGURATION request.                                     *
****************************************************************************
*)
$define USB_DisableEP1toEPn() GoSub __DisableEP1toEPn

__DisableEP1toEPn:
$ifdef _UEP8
    wUSB_FSR0 = VarPtr UEP1
    WREG = 15
    Repeat
        Clear POSTINC0
        Dec WREG
    Until STATUSBits_Z = True
$else
    UEP1 = 0
    UEP2 = 0
    UEP3 = 0
    UEP4 = 0
    UEP5 = 0
    UEP6 = 0
    UEP7 = 0
$endif
    Return

    Declare Bank_Resets = Off
(*
****************************************************************************
* Name    : USB_BufferReady                                                *
* Require : IN Endpoint: Buffer is loaded and ready to be sent.            *
*         : OUT Endpoint: Buffer is free to be written to by SIE.          *
* Input   : buffer_dsc: Root name of the buffer descriptor group. i.e.     *
*         : ep0Bo, ep1Bi, ... Declared in usbmmap.c. Names can be          *
*         : remapped for readability, see examples in usbcfg.h             *
* Purpose : This macro should be called each time after:                   *
*         :         1. A non-EP0 IN endpoint buffer is populated with data *
*         :         2. A non-EP0 OUT endpoint buffer is read.              *
*         :         This macro turns the buffer ownership to SIE for       *
*         :         servicing. It also toggles the DTS bit for             *
*         :         synchronization.                                       *
****************************************************************************
*)
$define USB_BufferReady(pBuffer_dsc) '
    pBuffer_dsc.Byte0 = pBuffer_dsc.Byte0 & _DTSMASK '
    pBuffer_dsc.DTS = ~pBuffer_dsc.DTS               '
    pBuffer_dsc.Byte0 = pBuffer_dsc.Byte0 | _USIE    '
    pBuffer_dsc.Byte0 = pBuffer_dsc.Byte0 | _DTSEN

(*
****************************************************************************
* Name    : mUSBCheckAdrPendingState                                       *
* Purpose : Specialised checking routine, it checks if the device is in    *
*         : the ADDRESS PENDING STATE and services it if it is.            *
****************************************************************************
*)
$define USB_CheckAdrPendingState() GoSub __mUSBCheckAdrPendingState

__mUSBCheckAdrPendingState:
    If bUSB_Device_State = ADR_PENDING_STATE Then
        UADDR = SetupPkt_bDevADR
        If UADDR > 0 Then
            bUSB_Device_State = ADDRESS_STATE
        Else
            bUSB_Device_State = DEFAULT_STATE
        EndIf
    EndIf
    Return
'---------------------------------------------------------------------------
    Declare Bank_Resets = On
$ifdef USB_USE_HID
    Include "USB_HID.inc"           ' Include the HID routines
$endif
    Declare Bank_Resets = Off
(*
****************************************************************************
* Name    : USBStdGetDscHandler                                            *
* Purpose : This routine handles the STANDARD GET_DESCRIPTOR request. It   *
*         : utilises tables dynamically looks up descriptor size. This     *
*         : routine should never have to be modified if the tables in      *
*         : USBdsc.c are declared correctly.                               *
****************************************************************************
*)
$define USB_StdGetDscHandler() GoSub __USBStdGetDscHandler

__USBStdGetDscHandler:
    If SetupPkt_bmRequestType = $80 Then
        bCtrl_Trf_Session_Owner = MUID_USB9
        Select SetupPkt_bDscType
            Case USB_DESCRIPTOR_DEVICE
                wSrc_Rom = VarPtr device_dsc                           ' Get the Address of the device_dsc table
                wUSB_TBLPTR = wSrc_Rom                                 ' Point to data count
            Case USB_DESCRIPTOR_CONFIGURATION
                wSrc_Rom = CRead16 USB_CD_Pointer[SetupPkt_bDscIndex]
                wUSB_TBLPTR = wSrc_Rom + 2                              ' Point to data count
            Case USB_DESCRIPTOR_STRING
                wSrc_Rom = CRead16 USB_SD_Pointer[SetupPkt_bDscIndex]
                wUSB_TBLPTR = wSrc_Rom                                  ' Point to  data count
        EndSelect
        Tblrd*+
        USB_Count = TABLAT                                              ' Get the size of the table into USB_Count
        bUSB_Stat.ctrl_trf_mem = _ROM_                                  ' Set memory type
    EndIf
    Return
(*
****************************************************************************
* Name    : USBStdSetCfgHandler                                            *
* Purpose : This routine first disables all endpoints by clearing UEP      *
*         : registers. It then configures (initialises) endpoints specified*
*         : in the modifiable section.                                     *
****************************************************************************
*)
$define USB_StdSetCfgHandler() GoSub __USBStdSetCfgHandler

__USBStdSetCfgHandler:
    bCtrl_Trf_Session_Owner = MUID_USB9
    USB_DisableEP1toEPn()
    bUSB_alt_intf = 0
    bUSB_Active_Cfg = SetupPkt_bCfgValue
    If SetupPkt_bCfgValue = 0 Then
        bUSB_Device_State = ADDRESS_STATE
    Else
        bUSB_Device_State = CONFIGURED_STATE
        '
        ' Modifiable Section
        '
        $ifdef USB_USE_HID
        HID_InitEP()
        $endif
        '
        ' End modifiable section
        '
    EndIf
    Return
(*
****************************************************************************
* Name    : USBStdGetStatusHandler                                         *
* Purpose : This routine handles the STANDARD GET_STATUS request           *
****************************************************************************
*)
$define USB_StdGetStatusHandler() GoSub __USBStdGetStatusHandler

__USBStdGetStatusHandler:
    CtrlTrfData_Byte0 = 0                       ' Initialise content
    CtrlTrfData_Byte1 = 0
    WREG = mGetRecipient()
    Select WREG
        Case RCPT_DEV
            bCtrl_Trf_Session_Owner = MUID_USB9
            '
            ' _byte0: bit0: Self-Powered Status [0] Bus-Powered [1] Self-Powered
            '         bit1: RemoteWakeup        [0] Disabled    [1] Enabled
            '
            $ifdef USB_SELF_POWER
                CtrlTrfData_Byte0.0 = 1
                If bUSB_Stat.RemoteWakeup = 1 Then
                    CtrlTrfData_Byte0.1 = 1
                EndIf
            $endif
        Case RCPT_INTF                          ' No data to update
            bCtrl_Trf_Session_Owner = MUID_USB9
        Case RCPT_EP
            bCtrl_Trf_Session_Owner = MUID_USB9
            '
            ' byte0: bit0: Halt Status [0] Not Halted [1] Halted
            '
            wUSB_Temp = VarPtr ep0Bo
            WREG = mGetEPNum()
            wUSB_FSR1 = WREG * 8
            wUSB_FSR1 = wUSB_FSR1 + wUSB_Temp
            WREG = SetupPkt_EPDir
            WREG = WREG * 4
            wUSB_FSR1 = wUSB_FSR1 + WREG
            WREG = POSTINC1 & _BSTALL           ' Use _BSTALL as a bit mask
            If WREG <> 0 Then
                CtrlTrfData_Byte0 = 1           ' Set bit0
            EndIf
    EndSelect
    If bCtrl_Trf_Session_Owner = MUID_USB9 Then
        wSrc_Ram = VarPtr bCtrlTrfData          ' Set Source
        bUSB_Stat.ctrl_trf_mem = _RAM_          ' Set memory type
        USB_Count.Byte0 = 2                     ' Set data count
    EndIf
    Return
(*
****************************************************************************
* Name    : USBStdFeatureReqHandler                                        *
* Purpose : This routine handles the STANDARD SET & clear FEATURES requests*
****************************************************************************
*)
$define USB_StdFeatureReqHandler() GoSub __USBStdFeatureReqHandler

__USBStdFeatureReqHandler:
    If SetupPkt_bFeature = DEVICE_REMOTE_WAKEUP Then
        WREG = mGetRecipient()
        If WREG = RCPT_DEV Then
            bCtrl_Trf_Session_Owner = MUID_USB9
            If SetupPkt_bRequest = SET_FEATURE Then
                bUSB_Stat.RemoteWakeup = 1
            Else
                bUSB_Stat.RemoteWakeup = 0
            EndIf
        EndIf
    EndIf
    If SetupPkt_bFeature = ENDPOINT_HALT Then
        WREG = mGetRecipient()
        If WREG = RCPT_EP Then
            WREG = mGetEPNum()
            If WREG <> 0 Then
                bCtrl_Trf_Session_Owner = MUID_USB9
                '
                ' Must do address calculation here
                '
                wUSB_Temp = VarPtr ep0Bo
                WREG = mGetEPNum()
                wUSB_FSR0 = WREG * 8
                wUSB_FSR0 = wUSB_FSR0 + wUSB_Temp
                WREG = SetupPkt_EPDir
                WREG = WREG * 4
                wUSB_FSR0 = wUSB_FSR0 + WREG

                If SetupPkt_bRequest = SET_FEATURE Then
                    POSTINC0 = _USIE | _BSTALL
                Else
                    If SetupPkt_EPDir = 1 Then      ' IN
                        POSTINC0 = _UCPU
                    Else
                        POSTINC0 = _USIE | _DAT0 | _DTSEN
                    EndIf
                EndIf
            EndIf
        EndIf
    EndIf
    Return
(*
****************************************************************************
* Name    : USBCheckStdRequest                                             *
* Purpose : This routine checks the setup data packet to see if it knows   *
*         : how to handle it                                               *
****************************************************************************
*)
$define USB_CheckStdRequest() GoSub __USBCheckStdRequest

__USBCheckStdRequest:
    WREG = SetupPkt_RequestType_f2 >> 5     ' \ GetRequestType
    WREG = WREG & %00000011                 ' /
    If WREG <> STANDARD Then Return
    Select SetupPkt_bRequest
        Case SET_Adr
            bCtrl_Trf_Session_Owner = MUID_USB9
            bUSB_Device_State = ADR_PENDING_STATE   ' Update state only
            '
            ' See USB_CtrlTrfInHandler for the next step
            '
        Case GET_DSC
            USB_StdGetDscHandler()
        Case SET_CFG
            USB_StdSetCfgHandler()
        Case GET_CFG
            bCtrl_Trf_Session_Owner = MUID_USB9
            wSrc_Ram = VarPtr bUSB_Active_Cfg       ' Set Source
            bUSB_Stat.ctrl_trf_mem = _RAM_          ' Set memory type
            USB_Count.Byte0 = 1                     ' Set data count
        Case GET_STATUS
            USB_StdGetStatusHandler()
        Case CLR_FEATURE
            USB_StdFeatureReqHandler()
        Case CLR_FEATURE
            USB_StdFeatureReqHandler()
        Case GET_INTF
            bCtrl_Trf_Session_Owner = MUID_USB9
            wSrc_Ram = VarPtr bUSB_alt_intf          ' \ Set source
            wSrc_Ram = wSrc_Ram + SetupPkt_bIntfID   ' /
            bUSB_Stat.ctrl_trf_mem = _RAM_           ' Set memory type
            USB_Count.Byte0 = 1                     ' Set data count
        Case SET_INTF
            bCtrl_Trf_Session_Owner = MUID_USB9
            'bUSB_alt_intf[SetupPkt_bIntfID] = SetupPkt_bAltID
            bUSB_alt_intf = SetupPkt_bAltID
        'Case SET_DSC
    EndSelect
    Return
(*
*****************************************************************************
* Name    : USBCtrlTrfTxService                                             *
* Require : pSrc, USB_Count, and bUSB_Stat.ctrl_trf_mem are setup properly.*
* Purpose : This routine should be called from only two places. One from    *
*         : USBCtrlEPServiceComplete() and one from USBCtrlTrfInHandler().  *
*         : It takes care of managing a transfer over multiple USB          *
*         : transactions. This routine works with isochronous endpoint      *
*         : larger than 256 bytes and is shown here as an example of how to *
*         : deal with BC9 and BC8. IN reality, a control endpoint can never *
*         : be larger than 64 bytes.                                        *
*****************************************************************************
*)
$define USB_CtrlTrfTxService() GoSub __USBCtrlTrfTxService

__USBCtrlTrfTxService:
' This code executes in the Data stage of a Control Read transfer.
'
' First, have to figure out how many bytes of data to send.
'
    If USB_Count < USB_EP0_BUFF_SIZE Then
        wUSB_BytesToSend = USB_Count
    Else
        wUSB_BytesToSend = USB_EP0_BUFF_SIZE
    EndIf
'
' Next, load the number of bytes to send to BC9..0 in buffer descriptor
'
    ep0Bi_Stat.BC9 = 0
    ep0Bi_Stat.BC8 = 0
    ep0Bi_Stat = ep0Bi_Stat | wUSB_BytesToSend.Byte1
    ep0Bi_Cnt = wUSB_BytesToSend.Byte0
    USB_Count = USB_Count - wUSB_BytesToSend        ' Subtract the number of bytes just about to be sent from the total.
    wDest_Ram = VarPtr bCtrlTrfData                 ' Set destination pointer
    If bUSB_Stat.ctrl_trf_mem = _ROM_ Then          ' Determine type of memory source
        wUSB_TBLPTR = wSrc_Rom
        wUSB_FSR0 = wDest_Ram
        wSrc_Rom = wSrc_Rom + wUSB_BytesToSend
        While wUSB_BytesToSend <> 0
            Tblrd*+
            POSTINC0 = TABLAT
            Dec wUSB_BytesToSend
        Wend
    Else                                            ' RAM
        wUSB_FSR1 = wSrc_Ram
        wUSB_FSR0 = wDest_Ram
        wSrc_Ram = wSrc_Ram + wUSB_BytesToSend
        While wUSB_BytesToSend > 0
            POSTINC0 = POSTINC1
            Dec wUSB_BytesToSend
        Wend
    EndIf
    Return
(*
****************************************************************************
* Name    : USBCtrlEPServiceComplete                                       *
* Purpose : This routine wrap up the ramaining tasks IN servicing a Setup  *
*         : Request. Its main task is to set the endpoint controls         *
*         : appropriately for a given situation. See code below.           *
*         : There are three main scenarios:                                *
*         :   a) There was no handler for the Request, IN this case a STALL*
*         :      should be sent OUT.                                       *
*         :   b) The host has requested a read control transfer, endpoints *
*         :      are required to be setup IN a specific way.               *
*         :   c) The host has requested a write control transfer, or a     *
*         :      control data stage is not required, endpoints are required*
*         :      to be setup IN a specific way.                            *
*         : Packet processing is resumed by clearing PKTDIS bit.           *
****************************************************************************
*)
$define USB_CtrlEPServiceComplete() GoSub __USBCtrlEPServiceComplete

__USBCtrlEPServiceComplete:
'
' PKTDIS bit is set when a Setup Transaction is received.
' Clear before modifying ep0Bi_Stat or ep0Bo_Stat.
'
    UCONbits_PKTDIS = 0
    If bCtrl_Trf_Session_Owner = MUID_NULL Then
        '
        ' If no one knows how to service this request then stall.
        ' Must also prepare EP0 to receive the next SETUP transaction.
        '
        ep0Bo_Cnt = USB_EP0_BUFF_SIZE
        ep0Bo_Adr = VarPtr SetupPkt
        ep0Bo_Stat = _USIE | _BSTALL
        ep0Bi_Stat = _USIE | _BSTALL
    '
    ' A module has claimed ownership of the control transfer session.
    '
    Else
        If SetupPkt_DataDir = DEV_TO_HOST Then
            If SetupPkt_wLength < USB_Count Then
                USB_Count = SetupPkt_wLength
            EndIf
            USB_CtrlTrfTxService()
            bCtrl_Trf_State = CTRL_TRF_TX
            '
            ' Control Read:
            ' <SETUP[0]><IN[1]><IN[0]>...<OUT[1]> | <SETUP[0]>
            ' 1. Prepare out EP to respond to early termination

            ' Note:
            ' If something went wrong during the control transfer,
            ' the last status stage may not be sent by the host.
            ' When this happens, two different things could happen
            ' depending on the host.
            ' a) The host could send out a Reset.
            ' b) The host could send out a new Setup transaction without sending a Reset first.
            ' To properly handle case (b), the out EP must be setup
            ' to receive either a zero length OUT transaction, or a
            ' new Setup transaction.

            ' Since the Setup transaction requires the DTS bit to be
            ' DAT0 while the zero length OUT status requires the DTS
            ' bit to be DAT1, the DTS bit check by the hardware should
            ' be disabled. This way the SIE could accept either of
            ' the two transactions.

            ' Furthermore, the Cnt byte should be set to prepare for
            ' the Setup data (8-byte or more), and the buffer address
            ' should be pointed to SetupPkt.
            '
            ep0Bo_Cnt = USB_EP0_BUFF_SIZE
            ep0Bo_Adr = VarPtr SetupPkt
            ep0Bo_Stat = _USIE                  ' Note: DTSEN is 0!
            '
            ' 2. Prepare in EP to transfer data, Cnt should have
            '    been initialised by responsible request owner.
            '
            ep0Bi_Adr = VarPtr bCtrlTrfData
            ep0Bi_Stat = _USIE | _DAT1 | _DTSEN
        Else
            bCtrl_Trf_State = CTRL_TRF_RX
            '
            ' Control Write:
            ' <SETUP[0]><OUT[1]><OUT[0]>...<IN[1]> | <SETUP[0]>
            ' 1. Prepare in EP to respond to early termination
            '    This is the same as a Zero Length Packet Response
            '    for control transfer without a data stage
            '
            ep0Bi_Cnt = 0
            ep0Bi_Stat = _USIE | _DAT1 | _DTSEN
            '
            ' Prepare out EP to receive data.
            '
            ep0Bo_Cnt = USB_EP0_BUFF_SIZE
            ep0Bo_Adr = VarPtr bCtrlTrfData
            ep0Bo_Stat = _USIE | _DAT1 | _DTSEN
        EndIf
    EndIf
    Return
(*
****************************************************************************
* Name    : USBCtrlTrfSetupHandler                                         *
* Require : SetupPkt buffer is loaded with valid USB Setup Data            *
* Purpose : This routine is a task dispatcher and has 3 stages.            *
*         :      1. It initialises the control transfer state machine.     *
*         :      2. It calls on each of the module that may know how to    *
*         :         service the Setup Request from the host.               *
*         :         module Example: USB9, HID, CDC, MSD, ...               *
*         :         as new classes are added, ClassReqHandler table IN     *
*         :         USBdsc.c should be updated to call all available       *
*         :         Class handlers.                                        *
*         :      3. Once each of the modules has had a chance to check if  *
*         :         it is responsible for servicing the request, stage 3   *
*         :         then checks direction of the transfer to determine how *
*         :         to prepare EP0 for the control transfer.               *
*         :         Refer to USBCtrlEPServiceComplete() for more details.  *
* Note    : Microchip USB Firmware has three different states for the      *
*         : control transfer state machine:                                *
*         :      1. WAIT_SETUP                                             *
*         :      2. CTRL_TRF_TX                                            *
*         :      3. CTRL_TRF_RX                                            *
*         : A Control Transfer is composed of many USB transactions. When  *
*         : transferring data over multiple transactions, it is important  *
*         : to keep track of data source, data destination, and data count *
*         : These three parameters are stored IN pSrc,pDst, and USB_Count. *
*         : A flag is used to note if the data source is from ROM or RAM.  *
****************************************************************************
*)
$define USB_CtrlTrfSetupHandler() GoSub __USBCtrlTrfSetupHandler

__USBCtrlTrfSetupHandler:
'
' Stage 1
'
    bCtrl_Trf_State = WAIT_SETUP
    bCtrl_Trf_Session_Owner = MUID_NULL   ' Set owner to NULL
    USB_Count = 0
'
' Stage 2
'
    USB_CheckStdRequest()
'
' Modifiable Section
'
$ifdef USB_USE_HID
    USB_CheckHIDRequest()
$endif
'
' End Modifiable Section
'
' Stage 3
'
    USB_CtrlEPServiceComplete()
    Return
(*
**********************************************************************************
* Name    : USBCtrlTrfRxService                                                  *
* Require : pDst and USB_Count are setup properly. pSrc is always &bCtrlTrfData *
*         : bUSB_Stat.ctrl_trf_mem is always _RAM_. USB_Count should be set to  *
*         : 0 at the start of each control transfer.                             *
* Purpose : This routine is only partially complete. Check for new version       *
*         : of the firmware.                                                     *
**********************************************************************************
*)
$define USB_CtrlTrfRxService() GoSub __USBCtrlTrfRxService

__USBCtrlTrfRxService:
'
' This code executes in the Data stage of a Control Write transfer.
'
    wUSB_BytesToRead.Byte1 = ep0Bo_Stat & $03        ' Filter out last 2 bits
    wUSB_BytesToRead.Byte0 = ep0Bo_Cnt
    USB_Count = USB_Count + wUSB_BytesToRead         ' Accumulate total number of bytes read
    wSrc_Ram = VarPtr bCtrlTrfData
    While wUSB_BytesToRead <> 0
        wUSB_FSR1 = wSrc_Ram
        POSTINC0 = POSTINC1
        wUSB_FSR0 = wDest_Ram
        Inc wDest_Ram
        Inc wSrc_Ram
        Dec wUSB_BytesToRead
    Wend
    Return
(*
****************************************************************************
* Name    : USBPrepareForNextSetupTrf                                      *
* Purpose : The routine forces EP0 OUT to be ready for a new Setup         *
*         : transaction, and forces EP0 IN to be owned by CPU.             *
****************************************************************************
*)
$define USB_PrepareForNextSetupTrf() GoSub __USBPrepareForNextSetupTrf

__USBPrepareForNextSetupTrf:
    bCtrl_Trf_State = WAIT_SETUP            ' See USBctrltrf.h
    ep0Bo_Cnt = USB_EP0_BUFF_SIZE
    ep0Bo_Adr = VarPtr SetupPkt
    ep0Bo_Stat = _USIE | _DAT0 | _DTSEN     ' EP0 buff dsc init
    ep0Bi_Stat = _UCPU                      ' EP0 IN buffer initialization
    Return
(*
****************************************************************************
* Name    : USBCtrlTrfOutHandler                                           *
* Purpose : This routine handles an OUT transaction according to which     *
*         : control transfer state is currently active. Note that if the   *
*         : the control transfer was from host to device, the session owner*
*         : should be notified at the end of each OUT transaction to       *
*         : service the received data.                                     *
****************************************************************************
*)
$define USB_CtrlTrfOutHandler() GoSub __USBCtrlTrfOutHandler

__USBCtrlTrfOutHandler:
    If bCtrl_Trf_State = CTRL_TRF_RX Then
        USB_CtrlTrfRxService()
        If ep0Bo_Stat.DTS = 0 Then
            ep0Bo_Stat = _USIE | _DAT1 | _DTSEN
        Else
            ep0Bo_Stat = _USIE | _DAT0 | _DTSEN
        EndIf
    Else
        USB_PrepareForNextSetupTrf()
    EndIf
    Return
'
' *** USBCtrlTrfOutHandler Should go Here ***
'
(*
****************************************************************************
* Name    : USBCtrlTrfInHandler                                            *
* Purpose : This routine handles an IN transaction according to which      *
*         : control transfer state is currently active. A Set Address      *
*         : Request must not change the acutal address of the device until *
*         : the completion of the control transfer. The end of the control *
*         : transfer for Set Address. Request is an in transaction.        *
*         : Therefore it is necessary to service this unique situation when*
*         :  the condition is right. Macro mUSBCheckAdrPendingState is     *
*         : defined in USB9.h and its function is to specifically service  *
*         : this event.                                                    *
****************************************************************************
*)
$define USB_CtrlTrfInHandler() GoSub __USBCtrlTrfInHandler

__USBCtrlTrfInHandler:
'
' Direction of data packet: device to host.
'
    USB_CheckAdrPendingState()                      ' Must check if in ADR_PENDING_STATE
    If bCtrl_Trf_State = CTRL_TRF_TX Then
        '
        ' It's the Data stage of a Control Read transfer.
        '
        USB_CtrlTrfTxService()
        If ep0Bi_Stat.DTS = 0 Then
            ep0Bi_Stat = _USIE | _DAT1 | _DTSEN
        Else
            ep0Bi_Stat = _USIE | _DAT0 | _DTSEN
        EndIf
    Else                                            ' CTRL_TRF_RX
        '
        ' It's the Status stage of a Control Write transfer.
        '
        USB_PrepareForNextSetupTrf()
    EndIf
    Return
(*
****************************************************************************
* Name    : USBCtrlEPService                                               *
* Require : USTAT is loaded with a valid endpoint address                  *
* Purpose : USBCtrlEPService checks for three transaction types that it    *
*         : knows how to service and services them.                        *
*         :    1. EP0 SETUP                                                *
*         :    2. EP0 OUT                                                  *
*         :    3. EP0 IN                                                   *
*         : It ignores all other types (i.e. EP1, EP2, etc.)               *
****************************************************************************
*)
$define USB_CtrlEPService() GoSub __USBCtrlEPService

__USBCtrlEPService:
    If USTAT = EP00_OUT Then
        WREG = ep0Bo_Stat_PID >> 2                  ' Get USB PID
        WREG = WREG & $0F                           ' EP0 SETUP
        If WREG = SETUP_TOKEN Then
            USB_CtrlTrfSetupHandler()
        Else
            USB_CtrlTrfOutHandler()
        EndIf
    ElseIf USTAT = EP00_IN Then                     ' EP0 IN
        USB_CtrlTrfInHandler()
    EndIf
    Return
(*
****************************************************************************
* Name    : USBModuleEnable                                                *
* Purpose : This routine enables the USB module. An end designer should    *
*         : never have to call this routine manually. This routine should  *
*         : only be called from USB_CheckBusStatus().                      *
*         : Mask all USB interrupts                                        *
*         : Enable module & attach To bus                                  *
****************************************************************************
*)
$define USB_ModuleEnable() '
    UCON = 0               '
    UIE = 0                '
    UCONbits_USBEN = 1     '
    bUSB_Device_State = ATTACHED_STATE

(*
****************************************************************************
* Name    : USBModuleDisable                                               *
* Purpose : This routine disables the USB module. An end designer should   *
*         : never have to use this routine manually. This routine should   *
*         : only be called from USB_CheckBusStatus().                      *
*         : Masks all USB interrupts                                       *
****************************************************************************
*)
$define USB_ModuleDisable() '
    UCON = 0                '
    UIE = 0                 '
    bUSB_Device_State = DETACHED_STATE
(*
****************************************************************************
* Name    : USB_CheckBusStatus                                             *
* Purpose : This routine enables/disables the USB module by monitoring the *
*         : USB power signal.                                              *
****************************************************************************
*)
$define USB_CheckBusStatus() GoSub __USBCheckBusStatus

__USBCheckBusStatus:
'
' Bus Attachment & Detachment Detection
' USB_bus_sense is an i/o pin defined in io_cfg.h
'
    If UCONbits_USBEN = 0 Then
        USB_ModuleEnable()              ' If off, enable it
    EndIf
'
' After enabling the USB module, it takes some time for the voltage
' on the D+ or D- line to rise high enough to get out of the SE0 condition.
' The USB Reset interrupt should not be unmasked until the SE0 condition is cleared.
' This helps preventing the firmware from misinterpreting this unique event as a USB bus reset from the USB host.
'
    If bUSB_Device_State = ATTACHED_STATE Then
        If UCONbits_SE0 = 0 Then
            UIR = 0                     ' Clear all USB interrupts
            UIE = 0                     ' Mask all USB interrupts
            UIEbits_URSTIE = 1          ' Unmask RESET interrupt
            UIEbits_IDLEIE = 1          ' Unmask IDLE interrupt
            bUSB_Device_State = POWERED_STATE
        EndIf
    EndIf
    Clrwdt
    Return
(*
****************************************************************************
* Name    : USBSoftDetach                                                  *
* Purpose : The device will have to be re-enumerated to function again     *
*         : USBSoftDetach electrically disconnects the device from the bus *
*         : This is done by stop supplying VUSB voltage to pull-up resistor*
*         : The pull-down resistors on the host side will pull both        *
*         : differential signal lines low and the host registers the event *
*         : as a disconnect.                                               *
*         : Since the USB cable is not physically disconnected, the power  *
*         : supply through the cable can still be sensed by the device.    *
*         : The next time USB_CheckBusStatus() function is called, it will *
*         : reconnect the device back to the bus.                          *
****************************************************************************
*)
$define USB_SoftDetach() USB_ModuleDisable()

(*
****************************************************************************
* Name    : USBWakeFromSuspend                                             *
* Purpose :                                                                *
****************************************************************************
*)
$define USB_WakeFromSuspend() _USBWakeFromSuspend

_USBWakeFromSuspend Macro-
    GoSub __USBWakeFromSuspend
Endm

#ifMacro- _USBWakeFromSuspend
__USBWakeFromSuspend:
'
' If using clock switching, this is the place to restore the original clock frequency.
'
    UCONbits_SUSPND = 0
    UIEbits_ACTVIE = 0
    UIRbits_ACTVIF = 0
    Return
#endIfMacro-
(*
****************************************************************************
* Name    : USBProtocolResetHandler                                        *
* Require : A USB bus reset is received from the host                      *
* Purpose : Currently, this routine flushes any pending USB transactions.  *
*         : It empties out the USTAT FIFO. This action might not be        *
*         : desirable in some applications.                                *
*         : Once a USB bus reset is received from the host, this routine   *
*         : should be called. It resets the device address to zero,        *
*         : disables all non-EP0 endpoints, initialises EP0 to be ready for*
*         : default communication, clears all USB interrupt flags, unmasks *
*         : applicable USB interrupts, and reinitialises internal          *
*         : state-machine variables.                                       *
****************************************************************************
*)
$define USB_ProtocolResetHandler() GoSub __USBProtocolResetHandler

__USBProtocolResetHandler:
    UEIR = 0                        ' Clear all USB error flags
    UIR = 0                         ' Clears all USB interrupts
    UEIE = %10011111                ' Unmask all USB error interrupts
    UIE = %01111011                 ' Enable all interrupts except ACTVIE
    UADDR = 0                       ' Reset to default address

    USB_DisableEP1toEPn()           ' Reset all non-EP0 UEPn registers
    UEP0 = EP_CTRL | HSHK_EN        ' Init EP0 as a Ctrl EP
    While UIRbits_TRNIF = 1         ' Flush any pending transact...
        UIRbits_TRNIF = 0
    Wend
    UCONbits_PKTDIS = 0             ' Make sure packet processing is enabled
    USB_PrepareForNextSetupTrf()    ' Declared in USBctrltrf.c
    bUSB_Stat.RemoteWakeup = 0      ' Default status flag to disable
    bUSB_Active_Cfg = 0             ' Clear active configuration
    bUSB_Device_State = DEFAULT_STATE
    Return
'
' Note. Subroutine USB_RemoteWakeup should go here if required
'
(*
****************************************************************************
* Name    : USBSuspend                                                     *
* Purpose :                                                                *
****************************************************************************
*)
$define USB_Suspend() GoSub __USBSuspend

__USBSuspend:
' Note: Do not clear UIRbits.ACTVIF here!
' Reason:
' ACTVIF is only generated once an IDLEIF has been generated.
' This is a 1:1 ratio interrupt generation.
' For every IDLEIF, there will be only one ACTVIF regardless of
' the number of subsequent bus transitions.
'
' If the ACTIF is cleared here, a problem could occur when:
' [       IDLE       ][bus activity ->
' <--- 3 ms ----->     ^
'                ^     ACTVIF=1
'                IDLEIF=1
'  #           #           #           #   (#=Program polling flags)
'                          ^
'                          This polling loop will see both
'                          IDLEIF=1 and ACTVIF=1.
'                          However, the program services IDLEIF first
'                          because ACTIVIE=0.
'                          If this routine clears the only ACTIVIF,
'                          then it can never get out of the suspend
'                          mode.
'
    UIEbits_ACTVIE = 1                  ' Enable bus activity interrupt
    UIRbits_IDLEIF = 0
    UCONbits_SUSPND = 1                 ' Put USB module in power conserve
'
' At this point the microcontroller can go into sleep, idle, or switch to a slower clock, etc.
' *** Modifiable Section ***
'   PIR2bits.USBIF = 0
'   INTCONbits.RBIF = 0
'   PIE2bits.USBIE = 1                  ' Set USB wakeup source
'   INTCONbits.RBIE = 1                 ' Set sw2,3 wakeup source
'   sleep                               ' Goto sleep
'   If INTCONbits.RBIF = 1 then         ' Check if external stimulus
'      USB_RemoteWakeup()               ' If yes, attempt RWU
'   Endif
'   PIE2bits.USBIE = 0
'   INTCONbits.RBIE = 0
' *** End Modifiable Section ***
    Return
(*
****************************************************************************
* Name    : USB_SOF_Handler                                                *
* Purpose : The USB host sends OUT a SOF packet to full-speed devices every*
*         : 1 ms. This interrupt may be useful for isochronous pipes. End  *
*         : designers should implement callback routine as necessary.      *
****************************************************************************
*)
$define USB_SOF_Handler() GoSub __USB_SOF_Handler

__USB_SOF_Handler:
    UIRbits_SOFIF = 0
'
' if device is configured, but Attached flag is false, then we
' count SOF until x milliseconds has elapsed before setting the
' Attached flag to true - version 1.2...
'
' Modifiable Section
'
$ifdef USB_USE_HID
    If bUSB_Device_State = CONFIGURED_STATE Then
        If tUSB_Attached = False Then
            Inc USB_wTimeToConnect
            If USB_wTimeToConnect > USB_CONNECT_MS Then
                USB_wTimeToConnect = 0
                tUSB_Attached = True
            EndIf
        EndIf
    EndIf
$endif
'
' End Modifiable Section
'
    Return
(*
****************************************************************************
* Name    : USBStallHandler                                                *
* Require : A STALL packet is sent to the host by the SIE.                 *
* Purpose : The STALLIF is set anytime the SIE sends OUT a STALL packet    *
*         : regardless of which endpoint causes it. A Setup transaction    *
*         : overrides the STALL function. A stalled endpoint stops stalling*
*         : once it receives a setup packet. In this case, the SIE will    *
*         : accept the Setup packet and set the TRNIF flag to notify the   *
*         : firmware. STALL function for that particular endpoint pipe will*
*         : be automatically disabled (direction specific).                *
*         : There are a few reasons for an endpoint to be stalled.         *
*         :    1. When a non-supported USB request is received.            *
*         :       Example GET_DESCRIPTOR(DEVICE_QUALIFIER)                 *
*         :    2. When an endpoint is currently halted.                    *
*         :    3. When the device CLASS specifies that an endpoint must    *
*         :       stall in response to a specific event.                   *
*         : UEPn.EPSTALL can be scanned to see which endpoint causes the   *
*         : stall event.                                                   *
****************************************************************************
*)
$define USB_StallHandler() GoSub __USBStallHandler

__USBStallHandler:
'
' Does not really have to do anything here, even for the control endpoint.
' All BDs of Endpoint 0 are owned by SIE right now,
' but once a Setup Transaction is received, the ownership for EP0_OUT will be returned to CPU.
' When the Setup Transaction is serviced, the ownership for EP0_IN will then be forced back to CPU by firmware.
'
    If UEP0bits_EPSTALL = 1 Then
        USB_PrepareForNextSetupTrf()        ' Firmware work-around
        UEP0bits_EPSTALL = 0
    EndIf
    UIRbits_STALLIF = 0
    Return
(*
****************************************************************************
* Name    : USBErrorHandler                                                *
* Purpose : The purpose of this interrupt is mainly for debugging during   *
*         : development. Check UEIR to see which error causes the interrupt*
****************************************************************************
*)
$define USB_ErrorHandler() _USBErrorHandler

_USBErrorHandler Macro-
    GoSub __USBErrorHandler
Endm

#ifMacro- _USBErrorHandler
__USBErrorHandler:
    UIRbits_UERRIF = 0
    Return
#endIfMacro-
(*
****************************************************************************
* Name    : USBDriverService                                               *
* Purpose : This routine is the heart of this firmware. It manages all USB *
*         : interrupts. Device state transitions through the following     *
*         : stages DETACHED -> ATTACHED -> POWERED -> DEFAULT ->           *
*         : ADDRESS_PENDING -> ADDRESSED -> CONFIGURED -> READY            *
****************************************************************************
*)
$define USB_DriverService() GoSub __USBDriverService

__USBDriverService:
'
' Pointless to continue servicing if USB cable is not even attached.
'
    If bUSB_Device_State = DETACHED_STATE Then Return
'
' Task A: Service USB Activity Interrupt
'
    If UIRbits_ACTVIF = 1 Then
        If UIEbits_ACTVIE = 1 Then
            USB_WakeFromSuspend()
        EndIf
    EndIf
'
' Pointless to continue servicing if the device is in suspend mode.
'
    If UCONbits_SUSPND = 1 Then
        tUSB_Attached = False
        USB_wTimeToConnect = 0
        Return
    EndIf
'
' Task B: Service USB Bus Reset Interrupt.
' When bus reset is received during suspend, ACTVIF will be set first,
' once the UCONbits.SUSPND is clear, then the URSTIF bit will be asserted.
' This is why URSTIF is checked after ACTVIF.
' The USB reset flag is masked when the USB state is in
' DETACHED_STATE or ATTACHED_STATE, and therefore cannot cause a USB reset event during these two states.
'
    If UIRbits_URSTIF = 1 Then
        If UIEbits_URSTIE = 1 Then
            USB_ProtocolResetHandler()
        EndIf
    EndIf
'
' Task C: Service other USB interrupts
'
    If UIRbits_IDLEIF = 1 Then
        If UIEbits_IDLEIE = 1 Then
            USB_Suspend()
        EndIf
    EndIf
    If UIRbits_SOFIF = 1 Then
        If UIEbits_SOFIE = 1 Then
            USB_SOF_Handler()
        EndIf
    EndIf
    If UIRbits_STALLIF = 1 Then
        If UIEbits_STALLIE = 1 Then
            USB_StallHandler()
        EndIf
    EndIf
    If UIRbits_UERRIF = 1 Then
        If UIEbits_UERRIE = 1 Then
            USB_ErrorHandler()
        EndIf
    EndIf
'
' Pointless to continue servicing if the host has not sent a bus reset.
' Once bus reset is received, the device transitions into the default state and is ready for communication.
'
    If bUSB_Device_State < DEFAULT_STATE Then Return
'
' Task D: Servicing USB Transaction Complete Interrupt
'
    If UIRbits_TRNIF = 1 Then
        If UIEbits_TRNIE = 1 Then
            '
            ' USB_CtrlEPService only services transactions over EP0.
            ' It ignores all other EP transactions.
            '
            USB_CtrlEPService()
            '
            ' Other EP can be serviced later by responsible device class firmware.
            ' Each device driver knows when an out or in transaction is ready by checking the buffer ownership bit.
            ' An out EP should always be owned by SIE until the data is ready.
            ' An in EP should always be owned by CPU until the data is ready.
            ' Because of this logic, it is not necessary to save the USTAT value of non-EP0 transactions.
            '
            UIRbits_TRNIF = 0
        EndIf
    EndIf
    Return

    Declare Bank_Resets = On
OverUSBSubs:

